import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Brokerages.GDAX
import QuantConnect.Brokerages.GDAX.Messages
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Concurrent
import System.Collections.Generic

System_EventHandler = typing.Any


class GDAXBrokerageFactory(QuantConnect.Brokerages.BrokerageFactory):
    """Factory method to create GDAX Websockets brokerage"""

    @property
    def BrokerageData(self) -> System.Collections.Generic.Dictionary[str, str]:
        """provides brokerage connection data"""
        ...

    def __init__(self) -> None:
        """Factory constructor"""
        ...

    def Dispose(self) -> None:
        """Not required"""
        ...

    def GetBrokerageModel(self, orderProvider: QuantConnect.Securities.IOrderProvider) -> QuantConnect.Brokerages.IBrokerageModel:
        """
        The brokerage model
        
        :param orderProvider: The order provider
        """
        ...

    def CreateBrokerage(self, job: QuantConnect.Packets.LiveNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Interfaces.IBrokerage:
        """Create the Brokerage instance"""
        ...


class GDAXFill(System.Object):
    """Tracks fill messages"""

    @property
    def Order(self) -> QuantConnect.Orders.Order:
        """The Lean order"""
        ...

    @property
    def OrderId(self) -> int:
        """Lean orderId"""
        ...

    @property
    def TotalQuantity(self) -> float:
        """Total amount executed across all fills"""
        ...

    @property
    def OrderQuantity(self) -> float:
        """Original order quantity"""
        ...

    def __init__(self, order: QuantConnect.Orders.Order) -> None:
        """Creates instance of GDAXFill"""
        ...

    def Add(self, msg: QuantConnect.Brokerages.GDAX.Messages.Fill) -> None:
        """Adds a trade message"""
        ...


class AuthenticationToken(System.Object):
    """Contains data used for authentication"""

    @property
    def Key(self) -> str:
        """The key"""
        ...

    @Key.setter
    def Key(self, value: str):
        """The key"""
        ...

    @property
    def Signature(self) -> str:
        """The hashed signature"""
        ...

    @Signature.setter
    def Signature(self, value: str):
        """The hashed signature"""
        ...

    @property
    def Timestamp(self) -> str:
        """The timestamp"""
        ...

    @Timestamp.setter
    def Timestamp(self, value: str):
        """The timestamp"""
        ...

    @property
    def Passphrase(self) -> str:
        """The pass phrase"""
        ...

    @Passphrase.setter
    def Passphrase(self, value: str):
        """The pass phrase"""
        ...


class GDAXBrokerage(QuantConnect.Brokerages.BaseWebsocketsBrokerage):
    """Utility methods for GDAX brokerage"""

    SignHeader: str = "CB-ACCESS-SIGN"
    """Sign Header"""

    KeyHeader: str = "CB-ACCESS-KEY"
    """Key Header"""

    TimeHeader: str = "CB-ACCESS-TIMESTAMP"
    """Timestamp Header"""

    PassHeader: str = "CB-ACCESS-PASSPHRASE"
    """Passphrase header"""

    @property
    def IsConnected(self) -> bool:
        ...

    @property
    def FillSplit(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Brokerages.GDAX.GDAXFill]:
        ...

    @FillSplit.setter
    def FillSplit(self, value: System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Brokerages.GDAX.GDAXFill]):
        ...

    @property
    def _aggregator(self) -> QuantConnect.Data.IDataAggregator:
        """
        Data Aggregator
        
        This field is protected.
        """
        ...

    @property
    def ChannelNames(self) -> typing.List[str]:
        """This property is protected."""
        ...

    @typing.overload
    def GetAuthenticationToken(self, request: typing.Any) -> QuantConnect.Brokerages.GDAX.AuthenticationToken:
        """
        Creates an auth token and adds to the request
        
        :param request: the rest request
        :returns: a token representing the request params.
        """
        ...

    @typing.overload
    def GetAuthenticationToken(self, body: str, method: str, url: str) -> QuantConnect.Brokerages.GDAX.AuthenticationToken:
        """
        Creates an auth token to sign a request
        
        :param body: the request body as json
        :param method: the http method
        :param url: the request url
        """
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """Creates a new order"""
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """This operation is not supported"""
        ...

    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """Cancels an order"""
        ...

    def Connect(self) -> None:
        """Connects the client to the broker's remote servers"""
        ...

    def Disconnect(self) -> None:
        """Closes the websockets connection"""
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """Gets all orders not yet closed"""
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """Gets all open positions"""
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """Gets the total account cash balance"""
        ...

    def GetHistory(self, request: QuantConnect.Data.HistoryRequest) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.BaseData]:
        """
        Gets the history for the requested security
        
        :param request: The historical data request
        :returns: An enumerable of bars covering the span specified in the request.
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def __init__(self, wssUrl: str, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, passPhrase: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator) -> None:
        """
        Constructor for brokerage
        
        :param wssUrl: websockets url
        :param websocket: instance of websockets client
        :param restClient: instance of rest client
        :param apiKey: api key
        :param apiSecret: api secret
        :param passPhrase: pass phrase
        :param algorithm: the algorithm instance is required to retreive account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: consolidate ticks
        """
        ...

    def OnMessage(self, sender: typing.Any, e: QuantConnect.Brokerages.WebSocketMessage) -> None:
        """Wss message handler"""
        ...

    def GetTick(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> QuantConnect.Data.Market.Tick:
        """Retrieves a price tick for a given symbol"""
        ...

    def Subscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> None:
        """Creates websocket message subscriptions for the supplied symbols"""
        ...

    def PollTick(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> None:
        """Poll for new tick to refresh conversion rate of non-USD denomination"""
        ...

    def Unsubscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> bool:
        """Ends current subscriptions"""
        ...


class GDAXDataQueueHandler(QuantConnect.Brokerages.GDAX.GDAXBrokerage, QuantConnect.Interfaces.IDataQueueHandler):
    """An implementation of IDataQueueHandler for GDAX"""

    @property
    def ChannelNames(self) -> typing.List[str]:
        """
        The list of websocket channels to subscribe
        
        This property is protected.
        """
        ...

    def __init__(self, wssUrl: str, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, passPhrase: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator) -> None:
        """Initializes a new instance of the GDAXDataQueueHandler class"""
        ...

    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: System_EventHandler) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Sets the job we're subscribing for
        
        :param job: Job we're subscribing for
        """
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...





import datetime

import QuantConnect
import QuantConnect.Algorithm.Framework.Alphas
import QuantConnect.Algorithm.Framework.Alphas.Analysis
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.Alpha
import QuantConnect.Lean.Engine.Alphas
import QuantConnect.Lean.Engine.TransactionHandlers
import QuantConnect.Packets
import System
import System.Threading


class StatisticsInsightManagerExtension(System.Object, QuantConnect.Algorithm.Framework.Alphas.IInsightManagerExtension):
    """Manages alpha statistics responsbilities"""

    @property
    def Statistics(self) -> QuantConnect.AlphaRuntimeStatistics:
        """
        Gets the current statistics. The values are current as of the time specified
        in AlphaRuntimeStatistics.MeanPopulationScore and AlphaRuntimeStatistics.RollingAveragedPopulationScore
        """
        ...

    @property
    def RollingAverageIsReady(self) -> bool:
        """Gets whether or not the rolling average statistics is ready"""
        ...

    def __init__(self, accountCurrencyProvider: QuantConnect.Interfaces.IAccountCurrencyProvider, tradablePercentOfVolume: float = 0.01, period: int = 100, requireRollingAverageWarmup: bool = False) -> None:
        """
        Initializes a new instance of the StatisticsInsightManagerExtension class
        
        :param accountCurrencyProvider: The account currency provider
        :param tradablePercentOfVolume: Percent of volume of first bar used to estimate the maximum number of tradable shares. Defaults to 1%
        :param period: The period used for exponential smoothing of scores - this is a number of insights. Defaults to 100 insight predictions.
        :param requireRollingAverageWarmup: Specify true to force the population average scoring to warmup before plotting.
        """
        ...

    def OnInsightGenerated(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
        """
        Handles the IAlgorithm.InsightsGenerated event
        Increments total, long and short counters. Updates long/short ratio
        
        :param context: The newly generated insight context
        """
        ...

    def OnInsightClosed(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
        """
        Computes an estimated value for the insight. This is intended to be invoked at the end of the
        insight period, i.e, when now == insight.GeneratedTimeUtc + insight.Period;
        
        :param context: Context whose insight has just closed
        """
        ...

    def OnInsightAnalysisCompleted(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
        """
        Updates the specified statistics with the new scores
        
        :param context: Context whose insight has just completed analysis
        """
        ...

    def Step(self, frontierTimeUtc: datetime.datetime) -> None:
        """
        Invokes the manager at the end of the time step.
        
        :param frontierTimeUtc: The current frontier time utc
        """
        ...

    def InitializeForRange(self, algorithmStartDate: datetime.datetime, algorithmEndDate: datetime.datetime, algorithmUtcTime: datetime.datetime) -> None:
        """
        Allows the extension to initialize itself over the expected range
        
        :param algorithmStartDate: The start date of the algorithm
        :param algorithmEndDate: The end date of the algorithm
        :param algorithmUtcTime: The algorithm's current utc time
        """
        ...


class ChartingInsightManagerExtension(System.Object, QuantConnect.Algorithm.Framework.Alphas.IInsightManagerExtension):
    """Manages alpha charting responsibilities."""

    AlphaAssets: str = "Alpha Assets"
    """The string name used for the Alpha Assets chart"""

    @property
    def SampleInterval(self) -> datetime.timedelta:
        """
        Gets or sets the interval at which alpha charts are updated. This is in realtion to algorithm time.
        
        This property is protected.
        """
        ...

    @SampleInterval.setter
    def SampleInterval(self, value: datetime.timedelta):
        """
        Gets or sets the interval at which alpha charts are updated. This is in realtion to algorithm time.
        
        This property is protected.
        """
        ...

    def __init__(self, algorithm: QuantConnect.Interfaces.IAlgorithm, statisticsManager: QuantConnect.Lean.Engine.Alphas.StatisticsInsightManagerExtension) -> None:
        """
        Initializes a new instance of the ChartingInsightManagerExtension class
        
        :param algorithm: The algorithm instance. This is only used for adding the charts to the algorithm. We purposefully do not save a reference to avoid potentially inconsistent reads
        :param statisticsManager: Statistics manager used to access mean population scores for charting
        """
        ...

    def Step(self, frontierTimeUtc: datetime.datetime) -> None:
        """
        Invokes the manager at the end of the time step.
        Samples and plots insight counts and population score.
        
        :param frontierTimeUtc: The current frontier time utc
        """
        ...

    def InitializeForRange(self, algorithmStartDate: datetime.datetime, algorithmEndDate: datetime.datetime, algorithmUtcTime: datetime.datetime) -> None:
        """
        Invoked after IAlgorithm.Initialize has been called.
        Determines chart sample interval and initial sample times
        
        :param algorithmStartDate: The start date of the algorithm
        :param algorithmEndDate: The end date of the algorithm
        :param algorithmUtcTime: The algorithm's current utc time
        """
        ...

    def OnInsightGenerated(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
        """
        Handles the IAlgorithm.InsightsGenerated event.
        Keep daily and total count of insights by symbol
        
        :param context: The newly generated insight analysis context
        """
        ...

    def OnInsightClosed(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
        """
        NOP - Charting is more concerned with population vs individual insights
        
        :param context: Context whose insight has just completed analysis
        """
        ...

    def OnInsightAnalysisCompleted(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
        """
        NOP - Charting is more concerned with population vs individual insights
        
        :param context: Context whose insight has just completed analysis
        """
        ...


class DefaultAlphaHandler(System.Object, QuantConnect.Lean.Engine.Alpha.IAlphaHandler):
    """Default alpha handler that supports sending insights to the messaging handler, analyzing insights online"""

    class AlphaResultPacketSender(System.Object, QuantConnect.Algorithm.Framework.Alphas.IInsightManagerExtension, System.IDisposable):
        """
        Encapsulates routing finalized insights to the messaging handler
        
        This class is protected.
        """

        def __init__(self, job: QuantConnect.Packets.AlgorithmNodePacket, messagingHandler: QuantConnect.Interfaces.IMessagingHandler, interval: datetime.timedelta, maximumNumberOfInsightsPerPacket: int) -> None:
            """
            Initialize a new instance of AlphaResultPacketSender
            
            :param job: Algorithm job
            :param messagingHandler: Message handler to use
            :param interval: Timespan interval
            :param maximumNumberOfInsightsPerPacket: Limit on insights per packet
            """
            ...

        def OnInsightAnalysisCompleted(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
            """Enqueue finalized insights to be sent via the messaging handler"""
            ...

        def Step(self, frontierTimeUtc: datetime.datetime) -> None:
            """No operation"""
            ...

        def InitializeForRange(self, algorithmStartDate: datetime.datetime, algorithmEndDate: datetime.datetime, algorithmUtcTime: datetime.datetime) -> None:
            """No operation"""
            ...

        def OnInsightGenerated(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
            """No operation"""
            ...

        def OnInsightClosed(self, context: QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightAnalysisContext) -> None:
            """No operation"""
            ...

        def Dispose(self) -> None:
            """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
            ...

    @property
    def CancellationToken(self) -> System.Threading.CancellationToken:
        """
        The cancellation token that will be cancelled when requested to exit
        
        This property is protected.
        """
        ...

    @property
    def IsActive(self) -> bool:
        """Gets a flag indicating if this handler's thread is still running and processing messages"""
        ...

    @IsActive.setter
    def IsActive(self, value: bool):
        """Gets a flag indicating if this handler's thread is still running and processing messages"""
        ...

    @property
    def RuntimeStatistics(self) -> QuantConnect.AlphaRuntimeStatistics:
        """Gets the current alpha runtime statistics"""
        ...

    @RuntimeStatistics.setter
    def RuntimeStatistics(self, value: QuantConnect.AlphaRuntimeStatistics):
        """Gets the current alpha runtime statistics"""
        ...

    @property
    def AlgorithmId(self) -> str:
        """
        Gets the algorithm's unique identifier
        
        This property is protected.
        """
        ...

    @property
    def LiveMode(self) -> bool:
        """
        Gets whether or not the job is a live job
        
        This property is protected.
        """
        ...

    @property
    def Job(self) -> QuantConnect.Packets.AlgorithmNodePacket:
        """
        Gets the algorithm job packet
        
        This property is protected.
        """
        ...

    @Job.setter
    def Job(self, value: QuantConnect.Packets.AlgorithmNodePacket):
        """
        Gets the algorithm job packet
        
        This property is protected.
        """
        ...

    @property
    def Algorithm(self) -> QuantConnect.Interfaces.IAlgorithm:
        """
        Gets the algorithm instance
        
        This property is protected.
        """
        ...

    @Algorithm.setter
    def Algorithm(self, value: QuantConnect.Interfaces.IAlgorithm):
        """
        Gets the algorithm instance
        
        This property is protected.
        """
        ...

    @property
    def MessagingHandler(self) -> QuantConnect.Interfaces.IMessagingHandler:
        """
        Gets the confgured messaging handler for sending packets
        
        This property is protected.
        """
        ...

    @MessagingHandler.setter
    def MessagingHandler(self, value: QuantConnect.Interfaces.IMessagingHandler):
        """
        Gets the confgured messaging handler for sending packets
        
        This property is protected.
        """
        ...

    @property
    def InsightManager(self) -> QuantConnect.Algorithm.Framework.Alphas.Analysis.IInsightManager:
        """
        Gets the insight manager instance used to manage the analysis of algorithm insights
        
        This property is protected.
        """
        ...

    @InsightManager.setter
    def InsightManager(self, value: QuantConnect.Algorithm.Framework.Alphas.Analysis.IInsightManager):
        """
        Gets the insight manager instance used to manage the analysis of algorithm insights
        
        This property is protected.
        """
        ...

    def Initialize(self, job: QuantConnect.Packets.AlgorithmNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm, messagingHandler: QuantConnect.Interfaces.IMessagingHandler, api: QuantConnect.Interfaces.IApi, transactionHandler: QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler) -> None:
        """
        Initializes this alpha handler to accept insights from the specified algorithm
        
        :param job: The algorithm job
        :param algorithm: The algorithm instance
        :param messagingHandler: Handler used for sending insights
        :param api: Api instance
        :param transactionHandler: Algorithms transaction handler
        """
        ...

    def AddInsightManagerCustomExtensions(self, statistics: QuantConnect.Lean.Engine.Alphas.StatisticsInsightManagerExtension) -> None:
        """
        Allows each alpha handler implementation to add there own optional extensions
        
        This method is protected.
        """
        ...

    def OnAfterAlgorithmInitialized(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Invoked after the algorithm's Initialize method was called allowing the alpha handler to check
        other things, such as sampling period for backtests
        
        :param algorithm: The algorithm instance
        """
        ...

    def ProcessSynchronousEvents(self) -> None:
        """Performs processing in sync with the algorithm's time loop to provide consisten reading of data"""
        ...

    def Exit(self) -> None:
        """Stops processing and stores insights"""
        ...

    def StoreInsights(self) -> None:
        """
        Save insight results to persistent storage
        
        This method is protected.
        """
        ...

    def CreateInsightManager(self) -> QuantConnect.Algorithm.Framework.Alphas.Analysis.IInsightManager:
        """
        Creates the InsightManager to manage the analysis of generated insights
        
        This method is protected.
        
        :returns: A new insight manager instance.
        """
        ...





import datetime
import typing

import QuantConnect
import QuantConnect.Algorithm
import QuantConnect.Data
import QuantConnect.Data.UniverseSelection
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.DataFeeds
import QuantConnect.Lean.Engine.Results
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Report
import QuantConnect.Securities
import System
import System.Collections.Generic

JsonConverter = typing.Any

QuantConnect_Report_NullResultValueTypeJsonConverter_T = typing.TypeVar("QuantConnect_Report_NullResultValueTypeJsonConverter_T")


class Report(System.Object):
    """Report class"""

    StatisticsFileName: str = "report-statistics.json"
    """File name for statistics"""

    def __init__(self, name: str, description: str, version: str, backtest: QuantConnect.Packets.BacktestResult, live: QuantConnect.Packets.LiveResult, pointInTimePortfolioDestination: str = None) -> None:
        """
        Create beautiful HTML and PDF Reports based on backtest and live data.
        
        :param name: Name of the strategy
        :param description: Description of the strategy
        :param version: Version number of the strategy
        :param backtest: Backtest result object
        :param live: Live result object
        :param pointInTimePortfolioDestination: Point in time portfolio json output base filename
        """
        ...

    def Compile(self, html: str, reportStatistics: str) -> None:
        """Compile the backtest data into a report"""
        ...


class DeedleUtil(System.Object):
    """Utility extension methods for Deedle series/frames"""

    @staticmethod
    def CumulativeSum(input: typing.Any) -> typing.Any:
        """
        Calculates the cumulative sum for the given series
        
        :param input: Series to calculate cumulative sum for
        :returns: Cumulative sum in series form.
        """
        ...

    @staticmethod
    def CumulativeProduct(input: typing.Any) -> typing.Any:
        """
        Calculates the cumulative product of the series. This is equal to the python pandas method: `df.cumprod()`
        
        :param input: Input series
        :returns: Cumulative product.
        """
        ...

    @staticmethod
    def CumulativeMax(input: typing.Any) -> typing.Any:
        """Calculates the cumulative max of the series. This is equal to the python pandas method: `df.cummax()`."""
        ...

    @staticmethod
    def PercentChange(input: typing.Any) -> typing.Any:
        """
        Calculates the percentage change from the previous value to the current
        
        :param input: Series to calculate percentage change for
        :returns: Percentage change in series form.
        """
        ...

    @staticmethod
    def CumulativeReturns(input: typing.Any) -> typing.Any:
        """
        Calculates the cumulative returns series of the given input equity curve
        
        :param input: Equity curve series
        :returns: Cumulative returns over time.
        """
        ...

    @staticmethod
    def TotalReturns(input: typing.Any) -> float:
        """
        Calculates the total returns over a period of time for the given input
        
        :param input: Equity curve series
        :returns: Total returns over time.
        """
        ...

    @staticmethod
    def DropSparseColumnsAll(frame: typing.Any) -> typing.Any:
        """
        Drops sparse columns only if every value is `missing` in the column
        
        :param frame: Data Frame
        :returns: new Frame with sparse columns dropped.
        """
        ...

    @staticmethod
    def DropSparseRowsAll(frame: typing.Any) -> typing.Any:
        """
        Drops sparse rows if and only if every value is `missing` in the Frame
        
        :param frame: Data Frame
        :returns: new Frame with sparse rows dropped.
        """
        ...


class CrisisEvent(System.Enum):
    """Crisis Events"""

    DotCom = 0
    """DotCom bubble - https://en.wikipedia.org/wiki/Dot-com_bubble"""

    SeptemberEleventh = 1
    """September 11, 2001 attacks - https://en.wikipedia.org/wiki/September_11_attacks"""

    USHousingBubble2003 = 2
    """United States housing bubble - https://en.wikipedia.org/wiki/United_States_housing_bubble"""

    GlobalFinancialCrisis = 3
    """https://en.wikipedia.org/wiki/Financial_crisis_of_2007%E2%80%9308"""

    FlashCrash = 4
    """The flash crash of 2010 - https://en.wikipedia.org/wiki/2010_Flash_Crash"""

    FukushimaMeltdown = 5
    """Fukushima nuclear power plant meltdown - https://en.wikipedia.org/wiki/Fukushima_Daiichi_nuclear_disaster"""

    USDowngradeEuropeanDebt = 6
    """
    United States credit rating downgrade - https://en.wikipedia.org/wiki/United_States_federal_government_credit-rating_downgrades
    European debt crisis - https://en.wikipedia.org/wiki/European_debt_crisis
    """

    EurozoneSeptember2012 = 7
    """European debt crisis - https://en.wikipedia.org/wiki/European_debt_crisis"""

    EurozoneOctober2014 = 8
    """European debt crisis - https://en.wikipedia.org/wiki/European_debt_crisis"""

    MarketSellOff2015 = 9
    """2015-2016 market sell off https://en.wikipedia.org/wiki/2015%E2%80%9316_stock_market_selloff"""

    Recovery = 10
    """Crisis recovery (2010 - 2012)"""

    NewNormal = 11
    """2014 - 2019 market performance"""

    COVID19 = 12
    """COVID-19 pandemic market crash"""


class Crisis(System.Object):
    """Crisis events utility class"""

    Events: System.Collections.Generic.Dictionary[QuantConnect.Report.CrisisEvent, QuantConnect.Report.Crisis] = ...
    """Crisis events and pre-defined values"""

    @property
    def Start(self) -> datetime.datetime:
        """Start of the crisis event"""
        ...

    @property
    def End(self) -> datetime.datetime:
        """End of the crisis event"""
        ...

    @property
    def Name(self) -> str:
        """Name of the crisis"""
        ...

    def __init__(self, name: str, start: datetime.datetime, end: datetime.datetime) -> None:
        """
        Creates a new crisis instance with the given name and start/end date.
        
        :param name: Name of the crisis
        :param start: Start date of the crisis
        :param end: End date of the crisis
        """
        ...

    @staticmethod
    def FromCrisis(crisisEvent: QuantConnect.Report.CrisisEvent) -> QuantConnect.Report.Crisis:
        """
        Returns a pre-defined crisis event
        
        :param crisisEvent: Crisis Event
        :returns: Pre-defined crisis event.
        """
        ...

    @typing.overload
    def ToString(self) -> str:
        """Converts instance to string using the dates in the instance as start/end dates"""
        ...

    @typing.overload
    def ToString(self, start: datetime.datetime, end: datetime.datetime) -> str:
        """
        Converts instance to string using the provided dates
        
        :param start: Start date
        :param end: End date
        """
        ...


class Program(System.Object):
    """Lean Report creates a PDF strategy summary from the backtest and live json objects."""

    @staticmethod
    def Main(args: typing.List[str]) -> None:
        ...


class ResultsUtil(System.Object):
    """Utility methods for dealing with the Result objects"""

    @staticmethod
    def EquityPoints(result: QuantConnect.Result) -> System.Collections.Generic.SortedList[datetime.datetime, float]:
        """
        Get the equity chart points
        
        :param result: Result object to extract the chart points
        """
        ...

    @staticmethod
    def BenchmarkPoints(result: QuantConnect.Result) -> System.Collections.Generic.SortedList[datetime.datetime, float]:
        """
        Gets the points of the benchmark
        
        :param result: Backtesting or live results
        :returns: Sorted list keyed by date and value.
        """
        ...


class Rolling(System.Object):
    """Rolling window functions"""

    @staticmethod
    def Beta(equityCurve: typing.Any, benchmarkSeries: typing.Any, windowSize: int = 132) -> typing.Any:
        """
        Calculate the rolling beta with the given window size (in days)
        
        :param equityCurve: The equity curve you want to measure beta for
        :param benchmarkSeries: The benchmark/series you want to calculate beta with
        :param windowSize: Days/window to lookback
        :returns: Rolling beta.
        """
        ...

    @staticmethod
    def Sharpe(equityCurve: typing.Any, months: int, riskFreeRate: float = 0.0) -> typing.Any:
        """
        Get the rolling sharpe of the given series with a lookback of . The risk free rate is adjustable
        
        :param equityCurve: Equity curve to calculate rolling sharpe for
        :param months: Number of months to calculate the rolling period for
        :param riskFreeRate: Risk free rate
        :returns: Rolling sharpe ratio.
        """
        ...


class DrawdownPeriod(System.Object):
    """Represents a period of time where the drawdown ranks amongst the top N drawdowns."""

    @property
    def Start(self) -> datetime.datetime:
        """Start of the drawdown period"""
        ...

    @property
    def End(self) -> datetime.datetime:
        """End of the drawdown period"""
        ...

    @property
    def PeakToTrough(self) -> float:
        """Loss in percent from peak to trough"""
        ...

    @property
    def Drawdown(self) -> float:
        """Loss in percent from peak to trough - Alias for PeakToTrough"""
        ...

    def __init__(self, start: datetime.datetime, end: datetime.datetime, drawdown: float) -> None:
        """
        Creates an instance with the given start, end, and drawdown
        
        :param start: Start of the drawdown period
        :param end: End of the drawdown period
        :param drawdown: Max drawdown of the period
        """
        ...


class PointInTimePortfolio(System.Object):
    """Lightweight portfolio at a point in time"""

    class PointInTimeHolding(System.Object):
        """Holding of an asset at a point in time"""

        @property
        def Symbol(self) -> QuantConnect.Symbol:
            """Symbol of the holding"""
            ...

        @Symbol.setter
        def Symbol(self, value: QuantConnect.Symbol):
            """Symbol of the holding"""
            ...

        @property
        def HoldingsValue(self) -> float:
            """Value of the holdings of the asset. Can be negative if shorting an asset"""
            ...

        @HoldingsValue.setter
        def HoldingsValue(self, value: float):
            """Value of the holdings of the asset. Can be negative if shorting an asset"""
            ...

        @property
        def Quantity(self) -> float:
            """Quantity of the asset. Can be negative if shorting an asset"""
            ...

        @Quantity.setter
        def Quantity(self, value: float):
            """Quantity of the asset. Can be negative if shorting an asset"""
            ...

        @property
        def AbsoluteHoldingsValue(self) -> float:
            """Absolute value of the holdings."""
            ...

        @property
        def AbsoluteHoldingsQuantity(self) -> float:
            """Absolute value of the quantity"""
            ...

        def __init__(self, symbol: typing.Union[QuantConnect.Symbol, str], holdingsValue: float, holdingsQuantity: float) -> None:
            """
            Creates an instance of PointInTimeHolding, representing a holding at a given point in time
            
            :param symbol: Symbol of the holding
            :param holdingsValue: Value of the holding
            :param holdingsQuantity: Quantity of the holding
            """
            ...

    @property
    def Time(self) -> datetime.datetime:
        """Time that this point in time portfolio is for"""
        ...

    @Time.setter
    def Time(self, value: datetime.datetime):
        """Time that this point in time portfolio is for"""
        ...

    @property
    def TotalPortfolioValue(self) -> float:
        """The total value of the portfolio. This is cash + absolute value of holdings"""
        ...

    @TotalPortfolioValue.setter
    def TotalPortfolioValue(self, value: float):
        """The total value of the portfolio. This is cash + absolute value of holdings"""
        ...

    @property
    def Cash(self) -> float:
        """The cash the portfolio has"""
        ...

    @Cash.setter
    def Cash(self, value: float):
        """The cash the portfolio has"""
        ...

    @property
    def Order(self) -> QuantConnect.Orders.Order:
        """The order we just processed"""
        ...

    @Order.setter
    def Order(self, value: QuantConnect.Orders.Order):
        """The order we just processed"""
        ...

    @property
    def Holdings(self) -> System.Collections.Generic.List[QuantConnect.Report.PointInTimePortfolio.PointInTimeHolding]:
        """A list of holdings at the current moment in time"""
        ...

    @Holdings.setter
    def Holdings(self, value: System.Collections.Generic.List[QuantConnect.Report.PointInTimePortfolio.PointInTimeHolding]):
        """A list of holdings at the current moment in time"""
        ...

    @property
    def Leverage(self) -> float:
        """Portfolio leverage - provided for convenience"""
        ...

    @Leverage.setter
    def Leverage(self, value: float):
        """Portfolio leverage - provided for convenience"""
        ...

    @typing.overload
    def __init__(self, order: QuantConnect.Orders.Order, portfolio: QuantConnect.Securities.SecurityPortfolioManager) -> None:
        """
        Creates an instance of the PointInTimePortfolio object
        
        :param order: Order applied to the portfolio
        :param portfolio: Algorithm portfolio at a point in time
        """
        ...

    @typing.overload
    def __init__(self, portfolio: QuantConnect.Report.PointInTimePortfolio, time: datetime.datetime) -> None:
        """
        Clones the provided portfolio
        
        :param portfolio: Portfolio
        :param time: Time
        """
        ...

    def NoEmptyHoldings(self) -> QuantConnect.Report.PointInTimePortfolio:
        """
        Filters out any empty holdings from the current Holdings
        
        :returns: Current object, but without empty holdings.
        """
        ...


class Metrics(System.Object):
    """Strategy metrics collection such as usage of funds and asset allocations"""

    @staticmethod
    @typing.overload
    def LeverageUtilization(equityCurve: typing.Any, orders: System.Collections.Generic.List[QuantConnect.Orders.Order]) -> typing.Any:
        """
        Calculates the leverage used from trades. The series used to call this extension function should
        be the equity curve with the associated Order objects that go along with it.
        
        :param equityCurve: Equity curve series
        :param orders: Orders associated with the equity curve
        :returns: Leverage utilization over time.
        """
        ...

    @staticmethod
    @typing.overload
    def LeverageUtilization(portfolios: System.Collections.Generic.List[QuantConnect.Report.PointInTimePortfolio]) -> typing.Any:
        """
        Gets the leverage utilization from a list of PointInTimePortfolio
        
        :param portfolios: Point in time portfolios
        :returns: Series of leverage utilization.
        """
        ...

    @staticmethod
    @typing.overload
    def AssetAllocations(equityCurve: typing.Any, orders: System.Collections.Generic.List[QuantConnect.Orders.Order]) -> typing.Any:
        """
        Calculates the portfolio's asset allocation percentage over time. The series used to call this extension function should
        be the equity curve with the associated Order objects that go along with it.
        
        :param equityCurve: Equity curve series
        :param orders: Orders associated with the equity curve
        """
        ...

    @staticmethod
    @typing.overload
    def AssetAllocations(portfolios: System.Collections.Generic.List[QuantConnect.Report.PointInTimePortfolio]) -> typing.Any:
        """
        Calculates the asset allocation percentage over time.
        
        :param portfolios: Point in time portfolios
        :returns: Series keyed by Symbol containing the percentage allocated to that asset over time.
        """
        ...

    @staticmethod
    @typing.overload
    def Exposure(equityCurve: typing.Any, orders: System.Collections.Generic.List[QuantConnect.Orders.Order], direction: QuantConnect.Orders.OrderDirection) -> typing.Any:
        """
        Strategy long/short exposure by asset class
        
        :param equityCurve: Equity curve
        :param orders: Orders of the strategy
        :param direction: Long or short
        :returns: Frame keyed by SecurityType and OrderDirection. Returns a Frame of exposure per asset per direction over time.
        """
        ...

    @staticmethod
    @typing.overload
    def Exposure(portfolios: System.Collections.Generic.List[QuantConnect.Report.PointInTimePortfolio], direction: QuantConnect.Orders.OrderDirection) -> typing.Any:
        """
        Strategy long/short exposure by asset class
        
        :param portfolios: Point in time portfolios
        :param direction: Long or short
        :returns: Frame keyed by SecurityType and OrderDirection. Returns a Frame of exposure per asset per direction over time.
        """
        ...


class NullResultValueTypeJsonConverter(typing.Generic[QuantConnect_Report_NullResultValueTypeJsonConverter_T], JsonConverter):
    """
    Removes null values in the Result object's x,y values so that
    deserialization can occur without exceptions.
    """

    def __init__(self) -> None:
        """Initialize a new instance of NullResultValueTypeJsonConverter{T}"""
        ...

    def CanConvert(self, objectType: typing.Type) -> bool:
        """
        Determine if this converter can convert a given type
        
        :param objectType: Object type to convert
        :returns: Always true.
        """
        ...

    def ReadJson(self, reader: typing.Any, objectType: typing.Type, existingValue: typing.Any, serializer: typing.Any) -> System.Object:
        """
        Read Json for conversion
        
        :returns: Resulting object.
        """
        ...

    def WriteJson(self, writer: typing.Any, value: typing.Any, serializer: typing.Any) -> None:
        """Write Json; Not implemented"""
        ...


class OrderTypeNormalizingJsonConverter(JsonConverter):
    """
    Normalizes the "Type" field to a value that will allow for
    successful deserialization in the OrderJsonConverter class.
    """

    def CanConvert(self, objectType: typing.Type) -> bool:
        """
        Determine if this Converter can convert a given object type
        
        :param objectType: Object type to convert
        :returns: True if assignable from Order.
        """
        ...

    def ReadJson(self, reader: typing.Any, objectType: typing.Type, existingValue: typing.Any, serializer: typing.Any) -> System.Object:
        """
        Read Json and convert
        
        :returns: Resulting Order.
        """
        ...

    def WriteJson(self, writer: typing.Any, value: typing.Any, serializer: typing.Any) -> None:
        """Write Json; Not implemented"""
        ...


class DrawdownCollection(System.Object):
    """Collection of drawdowns for the given period marked by start and end date"""

    @property
    def Start(self) -> datetime.datetime:
        """Starting time of the drawdown collection"""
        ...

    @Start.setter
    def Start(self, value: datetime.datetime):
        """Starting time of the drawdown collection"""
        ...

    @property
    def End(self) -> datetime.datetime:
        """Ending time of the drawdown collection"""
        ...

    @End.setter
    def End(self, value: datetime.datetime):
        """Ending time of the drawdown collection"""
        ...

    @property
    def Periods(self) -> int:
        """
        Number of periods to take into consideration for the top N drawdown periods.
        This will be the number of items contained in the Drawdowns collection.
        """
        ...

    @Periods.setter
    def Periods(self, value: int):
        """
        Number of periods to take into consideration for the top N drawdown periods.
        This will be the number of items contained in the Drawdowns collection.
        """
        ...

    @property
    def Drawdowns(self) -> System.Collections.Generic.List[QuantConnect.Report.DrawdownPeriod]:
        """Worst drawdowns encountered"""
        ...

    @Drawdowns.setter
    def Drawdowns(self, value: System.Collections.Generic.List[QuantConnect.Report.DrawdownPeriod]):
        """Worst drawdowns encountered"""
        ...

    @typing.overload
    def __init__(self, periods: int) -> None:
        """Creates an instance with a default collection (no items) and the top N worst drawdowns"""
        ...

    @typing.overload
    def __init__(self, strategySeries: typing.Any, periods: int) -> None:
        """
        Creates an instance from the given drawdowns and the top N worst drawdowns
        
        :param strategySeries: Equity curve with both live and backtesting merged
        :param periods: Periods this collection contains
        """
        ...

    @staticmethod
    def FromResult(backtestResult: QuantConnect.Packets.BacktestResult = None, liveResult: QuantConnect.Packets.LiveResult = None, periods: int = 5) -> QuantConnect.Report.DrawdownCollection:
        """
        Generate a new instance of DrawdownCollection from backtest and live Result derived instances
        
        :param backtestResult: Backtest result packet
        :param liveResult: Live result packet
        :param periods: Top N drawdown periods to get
        :returns: DrawdownCollection instance.
        """
        ...

    @staticmethod
    def NormalizeResults(backtestResult: QuantConnect.Packets.BacktestResult, liveResult: QuantConnect.Packets.LiveResult) -> typing.Any:
        """
        Normalizes the Series used to calculate the drawdown plots and charts
        
        :param backtestResult: Backtest result packet
        :param liveResult: Live result packet
        """
        ...

    @staticmethod
    def GetUnderwater(curve: typing.Any) -> typing.Any:
        """
        Gets the underwater plot for the provided curve.
        Data is expected to be the concatenated output of ResultsUtil.EquityPoints.
        
        :param curve: Equity curve
        """
        ...

    @staticmethod
    def GetUnderwaterFrame(curve: typing.Any) -> typing.Any:
        """
        Gets all the data associated with the underwater plot and everything used to generate it.
        Note that you should instead use GetUnderwater(Series{DateTime, double}) if you
        want to just generate an underwater plot. This is internally used to get the top N worst drawdown periods.
        
        :param curve: Equity curve
        :returns: Frame containing the following keys: "returns", "cumulativeMax", "drawdown".
        """
        ...

    @staticmethod
    def GetTopWorstDrawdowns(curve: typing.Any, periods: int) -> typing.Any:
        """
        Gets the top N worst drawdowns and associated statistics.
        Returns a Frame with the following keys: "duration", "cumulativeMax", "drawdown"
        
        :param curve: Equity curve
        :param periods: Top N worst periods. If this is greater than the results, we retrieve all the items instead
        :returns: Frame with the following keys: "duration", "cumulativeMax", "drawdown".
        """
        ...

    @staticmethod
    def GetDrawdownPeriods(curve: typing.Any, periods: int = 5) -> System.Collections.Generic.IEnumerable[QuantConnect.Report.DrawdownPeriod]:
        """
        Gets the given drawdown periods from the equity curve and the set periods
        
        :param curve: Equity curve
        :param periods: Top N drawdown periods to get
        :returns: Enumerable of DrawdownPeriod.
        """
        ...


class PortfolioLooperAlgorithm(QuantConnect.Algorithm.QCAlgorithm):
    """Fake algorithm that initializes portfolio and algorithm securities. Never ran."""

    def __init__(self, startingCash: float, orders: System.Collections.Generic.IEnumerable[QuantConnect.Orders.Order]) -> None:
        """
        Initialize an instance of PortfolioLooperAlgorithm
        
        :param startingCash: Starting algorithm cash
        :param orders: Orders to use
        """
        ...

    def FromOrders(self, orders: System.Collections.Generic.IEnumerable[QuantConnect.Orders.Order]) -> None:
        """
        Initializes all the proper Securities from the orders provided by the user
        
        :param orders: Orders to use
        """
        ...

    def Initialize(self) -> None:
        """Initialize this algorithm"""
        ...


class PortfolioLooper(System.Object, System.IDisposable):
    """
    Runs LEAN to calculate the portfolio at a given time from Order objects.
    Generates and returns PointInTimePortfolio objects that represents
    the holdings and other miscellaneous metrics at a point in time by reprocessing the orders
    as they were filled.
    """

    @property
    def Algorithm(self) -> QuantConnect.Report.PortfolioLooperAlgorithm:
        """
        QCAlgorithm derived class that sets up internal data feeds for
        use with crypto and forex data, as well as managing the SecurityPortfolioManager
        """
        ...

    @Algorithm.setter
    def Algorithm(self, value: QuantConnect.Report.PortfolioLooperAlgorithm):
        """
        QCAlgorithm derived class that sets up internal data feeds for
        use with crypto and forex data, as well as managing the SecurityPortfolioManager
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    @staticmethod
    def GetHistory(symbols: System.Collections.Generic.List[QuantConnect.Symbol], start: datetime.datetime, end: datetime.datetime, resolution: QuantConnect.Resolution) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Slice]:
        """
        Gets the history for the given symbols from the  to the
        
        :param symbols: Symbols to request history for
        :param start: Start date of history request
        :param end: End date of history request
        :param resolution: Resolution of history request
        :returns: Enumerable of slices.
        """
        ...

    @staticmethod
    def FromOrders(equityCurve: typing.Any, orders: System.Collections.Generic.IEnumerable[QuantConnect.Orders.Order], liveSeries: bool = False) -> System.Collections.Generic.IEnumerable[QuantConnect.Report.PointInTimePortfolio]:
        """
        Gets the point in time portfolio over multiple deployments
        
        :param equityCurve: Equity curve series
        :param orders: Orders
        :param liveSeries: Equity curve series originates from LiveResult
        :returns: Enumerable of PointInTimePortfolio.
        """
        ...


class MockDataFeed(System.Object, QuantConnect.Lean.Engine.DataFeeds.IDataFeed):
    """Fake IDataFeed"""

    @property
    def IsActive(self) -> bool:
        """Bool if the feed is active"""
        ...

    def Initialize(self, algorithm: QuantConnect.Interfaces.IAlgorithm, job: QuantConnect.Packets.AlgorithmNodePacket, resultHandler: QuantConnect.Lean.Engine.Results.IResultHandler, mapFileProvider: QuantConnect.Interfaces.IMapFileProvider, factorFileProvider: QuantConnect.Interfaces.IFactorFileProvider, dataProvider: QuantConnect.Interfaces.IDataProvider, subscriptionManager: QuantConnect.Lean.Engine.DataFeeds.IDataFeedSubscriptionManager, dataFeedTimeProvider: QuantConnect.Lean.Engine.DataFeeds.IDataFeedTimeProvider, channelProvider: QuantConnect.Interfaces.IDataChannelProvider) -> None:
        """
        Initialize the data feed
        This implementation does nothing
        """
        ...

    def CreateSubscription(self, request: QuantConnect.Data.UniverseSelection.SubscriptionRequest) -> QuantConnect.Lean.Engine.DataFeeds.Subscription:
        """
        Create Subscription
        
        :param request: Subscription request to use
        :returns: Always null.
        """
        ...

    def RemoveSubscription(self, subscription: QuantConnect.Lean.Engine.DataFeeds.Subscription) -> None:
        """
        Remove Subscription; Not implemented
        
        :param subscription: Subscription to remove
        """
        ...

    def Exit(self) -> None:
        """DataFeed Exit"""
        ...





import typing

import QuantConnect.Interfaces
import QuantConnect.Messaging
import QuantConnect.Notifications
import QuantConnect.Packets
import System

QuantConnect_Messaging_EventMessagingHandler_DebugEventRaised = typing.Any
QuantConnect_Messaging_EventMessagingHandler_SystemDebugEventRaised = typing.Any
QuantConnect_Messaging_EventMessagingHandler_LogEventRaised = typing.Any
QuantConnect_Messaging_EventMessagingHandler_RuntimeErrorEventRaised = typing.Any
QuantConnect_Messaging_EventMessagingHandler_HandledErrorEventRaised = typing.Any
QuantConnect_Messaging_EventMessagingHandler_BacktestResultEventRaised = typing.Any
QuantConnect_Messaging_EventMessagingHandler_ConsumerReadyEventRaised = typing.Any


class StreamingMessageHandler(System.Object, QuantConnect.Interfaces.IMessagingHandler):
    """Message handler that sends messages over tcp using NetMQ."""

    @property
    def HasSubscribers(self) -> bool:
        """
        Gets or sets whether this messaging handler has any current subscribers.
        This is not used in this message handler.  Messages are sent via tcp as they arrive
        """
        ...

    @HasSubscribers.setter
    def HasSubscribers(self, value: bool):
        """
        Gets or sets whether this messaging handler has any current subscribers.
        This is not used in this message handler.  Messages are sent via tcp as they arrive
        """
        ...

    def Initialize(self) -> None:
        """Initialize the messaging system"""
        ...

    def SetAuthentication(self, job: QuantConnect.Packets.AlgorithmNodePacket) -> None:
        """Set the user communication channel"""
        ...

    def SendNotification(self, notification: QuantConnect.Notifications.Notification) -> None:
        """
        Send any notification with a base type of Notification.
        
        :param notification: The notification to be sent.
        """
        ...

    def Send(self, packet: QuantConnect.Packets.Packet) -> None:
        """Send all types of packets"""
        ...

    def Transmit(self, packet: QuantConnect.Packets.Packet) -> None:
        """
        Send a message to the _server using ZeroMQ
        
        :param packet: Packet to transmit
        """
        ...

    def Dispose(self) -> None:
        """Dispose any resources used before destruction"""
        ...


class Messaging(System.Object, QuantConnect.Interfaces.IMessagingHandler):
    """Local/desktop implementation of messaging system for Lean Engine."""

    @property
    def HasSubscribers(self) -> bool:
        """
        This implementation ignores the  flag and
        instead will always write to the log.
        """
        ...

    @HasSubscribers.setter
    def HasSubscribers(self, value: bool):
        """
        This implementation ignores the  flag and
        instead will always write to the log.
        """
        ...

    def Initialize(self) -> None:
        """Initialize the messaging system"""
        ...

    def SetAuthentication(self, job: QuantConnect.Packets.AlgorithmNodePacket) -> None:
        """Set the messaging channel"""
        ...

    def Send(self, packet: QuantConnect.Packets.Packet) -> None:
        """Send a generic base packet without processing"""
        ...

    def SendNotification(self, notification: QuantConnect.Notifications.Notification) -> None:
        """Send any notification with a base type of Notification."""
        ...

    def Dispose(self) -> None:
        """Dispose of any resources"""
        ...


class EventMessagingHandler(System.Object, QuantConnect.Interfaces.IMessagingHandler):
    """Desktop implementation of messaging system for Lean Engine"""

    @property
    def HasSubscribers(self) -> bool:
        """
        Gets or sets whether this messaging handler has any current subscribers.
        When set to false, messages won't be sent.
        """
        ...

    @HasSubscribers.setter
    def HasSubscribers(self, value: bool):
        """
        Gets or sets whether this messaging handler has any current subscribers.
        When set to false, messages won't be sent.
        """
        ...

    @property
    def DebugEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_DebugEventRaised]:
        ...

    @DebugEvent.setter
    def DebugEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_DebugEventRaised]):
        ...

    @property
    def SystemDebugEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_SystemDebugEventRaised]:
        ...

    @SystemDebugEvent.setter
    def SystemDebugEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_SystemDebugEventRaised]):
        ...

    @property
    def LogEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_LogEventRaised]:
        ...

    @LogEvent.setter
    def LogEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_LogEventRaised]):
        ...

    @property
    def RuntimeErrorEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_RuntimeErrorEventRaised]:
        ...

    @RuntimeErrorEvent.setter
    def RuntimeErrorEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_RuntimeErrorEventRaised]):
        ...

    @property
    def HandledErrorEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_HandledErrorEventRaised]:
        ...

    @HandledErrorEvent.setter
    def HandledErrorEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_HandledErrorEventRaised]):
        ...

    @property
    def BacktestResultEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_BacktestResultEventRaised]:
        ...

    @BacktestResultEvent.setter
    def BacktestResultEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_BacktestResultEventRaised]):
        ...

    @property
    def ConsumerReadyEvent(self) -> typing.List[QuantConnect_Messaging_EventMessagingHandler_ConsumerReadyEventRaised]:
        ...

    @ConsumerReadyEvent.setter
    def ConsumerReadyEvent(self, value: typing.List[QuantConnect_Messaging_EventMessagingHandler_ConsumerReadyEventRaised]):
        ...

    def Initialize(self) -> None:
        """Initialize the Messaging System Plugin."""
        ...

    def LoadingComplete(self) -> None:
        """Set Loaded to true"""
        ...

    def SetAuthentication(self, job: QuantConnect.Packets.AlgorithmNodePacket) -> None:
        """Set the user communication channel"""
        ...

    def DebugEventRaised(self, packet: QuantConnect.Packets.DebugPacket) -> None:
        ...

    def SystemDebugEventRaised(self, packet: QuantConnect.Packets.SystemDebugPacket) -> None:
        ...

    def LogEventRaised(self, packet: QuantConnect.Packets.LogPacket) -> None:
        ...

    def RuntimeErrorEventRaised(self, packet: QuantConnect.Packets.RuntimeErrorPacket) -> None:
        ...

    def HandledErrorEventRaised(self, packet: QuantConnect.Packets.HandledErrorPacket) -> None:
        ...

    def BacktestResultEventRaised(self, packet: QuantConnect.Packets.BacktestResultPacket) -> None:
        ...

    def ConsumerReadyEventRaised(self) -> None:
        ...

    def Send(self, packet: QuantConnect.Packets.Packet) -> None:
        ...

    def SendNotification(self, notification: QuantConnect.Notifications.Notification) -> None:
        """
        Send any notification with a base type of Notification.
        
        :param notification: The notification to be sent.
        """
        ...

    def SendEnqueuedPackets(self) -> None:
        """Send any message with a base type of Packet that has been enqueued."""
        ...

    def OnDebugEvent(self, packet: QuantConnect.Packets.DebugPacket) -> None:
        """
        Raise a debug event safely
        
        This method is protected.
        """
        ...

    def OnSystemDebugEvent(self, packet: QuantConnect.Packets.SystemDebugPacket) -> None:
        """
        Raise a system debug event safely
        
        This method is protected.
        """
        ...

    def OnConsumerReadyEvent(self) -> None:
        """Handler for consumer ready code."""
        ...

    def OnLogEvent(self, packet: QuantConnect.Packets.LogPacket) -> None:
        """
        Raise a log event safely
        
        This method is protected.
        """
        ...

    def OnHandledErrorEvent(self, packet: QuantConnect.Packets.HandledErrorPacket) -> None:
        """
        Raise a handled error event safely
        
        This method is protected.
        """
        ...

    def OnRuntimeErrorEvent(self, packet: QuantConnect.Packets.RuntimeErrorPacket) -> None:
        """
        Raise runtime error safely
        
        This method is protected.
        """
        ...

    def OnBacktestResultEvent(self, packet: QuantConnect.Packets.BacktestResultPacket) -> None:
        """
        Raise a backtest result event safely.
        
        This method is protected.
        """
        ...

    def Dispose(self) -> None:
        """Dispose of any resources"""
        ...





import typing

import Internal.Runtime.CompilerServices
import System

Internal_Runtime_CompilerServices_Unsafe_AsPointer_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_AsPointer_T")
Internal_Runtime_CompilerServices_Unsafe_As_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_As_T")
Internal_Runtime_CompilerServices_Unsafe_As_TFrom = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_As_TFrom")
Internal_Runtime_CompilerServices_Unsafe_Add_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_Add_T")
Internal_Runtime_CompilerServices_Unsafe_AreSame_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_AreSame_T")
Internal_Runtime_CompilerServices_Unsafe_IsAddressGreaterThan_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_IsAddressGreaterThan_T")
Internal_Runtime_CompilerServices_Unsafe_IsAddressLessThan_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_IsAddressLessThan_T")
Internal_Runtime_CompilerServices_Unsafe_ReadUnaligned_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_ReadUnaligned_T")
Internal_Runtime_CompilerServices_Unsafe_WriteUnaligned_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_WriteUnaligned_T")
Internal_Runtime_CompilerServices_Unsafe_AddByteOffset_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_AddByteOffset_T")
Internal_Runtime_CompilerServices_Unsafe_Read_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_Read_T")
Internal_Runtime_CompilerServices_Unsafe_Write_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_Write_T")
Internal_Runtime_CompilerServices_Unsafe_AsRef_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_AsRef_T")
Internal_Runtime_CompilerServices_Unsafe_ByteOffset_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_ByteOffset_T")
Internal_Runtime_CompilerServices_Unsafe_IsNullRef_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_IsNullRef_T")
Internal_Runtime_CompilerServices_Unsafe_SkipInit_T = typing.TypeVar("Internal_Runtime_CompilerServices_Unsafe_SkipInit_T")


class Unsafe(System.Object):
    """This class has no documentation."""

    @staticmethod
    def AsPointer(value: Internal_Runtime_CompilerServices_Unsafe_AsPointer_T) -> typing.Any:
        """Returns a pointer to the given by-ref parameter."""
        ...

    @staticmethod
    def SizeOf() -> int:
        """Returns the size of an object of the given type parameter."""
        ...

    @staticmethod
    @typing.overload
    def As(value: typing.Any) -> Internal_Runtime_CompilerServices_Unsafe_As_T:
        """Casts the given object to the specified type, performs no dynamic type checking."""
        ...

    @staticmethod
    @typing.overload
    def As(source: Internal_Runtime_CompilerServices_Unsafe_As_TFrom) -> typing.Any:
        """Reinterprets the given reference as a reference to a value of type TTo."""
        ...

    @staticmethod
    @typing.overload
    def Add(source: Internal_Runtime_CompilerServices_Unsafe_Add_T, elementOffset: int) -> typing.Any:
        """Adds an element offset to the given reference."""
        ...

    @staticmethod
    @typing.overload
    def Add(source: Internal_Runtime_CompilerServices_Unsafe_Add_T, elementOffset: System.IntPtr) -> typing.Any:
        """Adds an element offset to the given reference."""
        ...

    @staticmethod
    @typing.overload
    def Add(source: typing.Any, elementOffset: int) -> typing.Any:
        """Adds an element offset to the given pointer."""
        ...

    @staticmethod
    def AreSame(left: Internal_Runtime_CompilerServices_Unsafe_AreSame_T, right: Internal_Runtime_CompilerServices_Unsafe_AreSame_T) -> bool:
        """Determines whether the specified references point to the same location."""
        ...

    @staticmethod
    def IsAddressGreaterThan(left: Internal_Runtime_CompilerServices_Unsafe_IsAddressGreaterThan_T, right: Internal_Runtime_CompilerServices_Unsafe_IsAddressGreaterThan_T) -> bool:
        """
        Determines whether the memory address referenced by  is greater than
        the memory address referenced by .
        """
        ...

    @staticmethod
    def IsAddressLessThan(left: Internal_Runtime_CompilerServices_Unsafe_IsAddressLessThan_T, right: Internal_Runtime_CompilerServices_Unsafe_IsAddressLessThan_T) -> bool:
        """
        Determines whether the memory address referenced by  is less than
        the memory address referenced by .
        """
        ...

    @staticmethod
    def InitBlockUnaligned(startAddress: int, value: int, byteCount: int) -> None:
        """
        Initializes a block of memory at the given location with a given initial value
        without assuming architecture dependent alignment of the address.
        """
        ...

    @staticmethod
    @typing.overload
    def ReadUnaligned(source: typing.Any) -> Internal_Runtime_CompilerServices_Unsafe_ReadUnaligned_T:
        """Reads a value of type T from the given location."""
        ...

    @staticmethod
    @typing.overload
    def ReadUnaligned(source: int) -> Internal_Runtime_CompilerServices_Unsafe_ReadUnaligned_T:
        """Reads a value of type T from the given location."""
        ...

    @staticmethod
    @typing.overload
    def WriteUnaligned(destination: typing.Any, value: Internal_Runtime_CompilerServices_Unsafe_WriteUnaligned_T) -> None:
        """Writes a value of type T to the given location."""
        ...

    @staticmethod
    @typing.overload
    def WriteUnaligned(destination: int, value: Internal_Runtime_CompilerServices_Unsafe_WriteUnaligned_T) -> None:
        """Writes a value of type T to the given location."""
        ...

    @staticmethod
    def AddByteOffset(source: Internal_Runtime_CompilerServices_Unsafe_AddByteOffset_T, byteOffset: System.IntPtr) -> typing.Any:
        """Adds an byte offset to the given reference."""
        ...

    @staticmethod
    @typing.overload
    def Read(source: typing.Any) -> Internal_Runtime_CompilerServices_Unsafe_Read_T:
        """Reads a value of type T from the given location."""
        ...

    @staticmethod
    @typing.overload
    def Read(source: int) -> Internal_Runtime_CompilerServices_Unsafe_Read_T:
        """Reads a value of type T from the given location."""
        ...

    @staticmethod
    @typing.overload
    def Write(destination: typing.Any, value: Internal_Runtime_CompilerServices_Unsafe_Write_T) -> None:
        """Writes a value of type T to the given location."""
        ...

    @staticmethod
    @typing.overload
    def Write(destination: int, value: Internal_Runtime_CompilerServices_Unsafe_Write_T) -> None:
        """Writes a value of type T to the given location."""
        ...

    @staticmethod
    @typing.overload
    def AsRef(source: typing.Any) -> typing.Any:
        """Reinterprets the given location as a reference to a value of type T."""
        ...

    @staticmethod
    @typing.overload
    def AsRef(source: Internal_Runtime_CompilerServices_Unsafe_AsRef_T) -> typing.Any:
        """Reinterprets the given location as a reference to a value of type T."""
        ...

    @staticmethod
    def ByteOffset(origin: Internal_Runtime_CompilerServices_Unsafe_ByteOffset_T, target: Internal_Runtime_CompilerServices_Unsafe_ByteOffset_T) -> System.IntPtr:
        """Determines the byte offset from origin to target from the given references."""
        ...

    @staticmethod
    def NullRef() -> typing.Any:
        """Returns a by-ref to type T that is a null reference."""
        ...

    @staticmethod
    def IsNullRef(source: Internal_Runtime_CompilerServices_Unsafe_IsNullRef_T) -> bool:
        """Returns if a given by-ref to type T is a null reference."""
        ...

    @staticmethod
    def SkipInit(value: Internal_Runtime_CompilerServices_Unsafe_SkipInit_T) -> None:
        """Bypasses definite assignment rules by taking advantage of out semantics."""
        ...





import datetime
import typing

import QuantConnect.Data
import QuantConnect.Data.Custom.Intrinio
import QuantConnect.Util
import System


class IntrinioDataTransformation(System.Enum):
    """TRanformation available for the Economic data."""

    Roc = 0
    """The rate of change"""

    AnnualyRoc = 1
    """Rate of change from Year Ago"""

    CompoundedAnnualRoc = 2
    """The compounded annual rate of change"""

    AnnualyCCRoc = 3
    """The continuously compounded annual rate of change"""

    CCRoc = 4
    """The continuously compounded rateof change"""

    Level = 5
    """The level, no transformation."""

    Ln = 6
    """The natural log"""

    Pc = 7
    """The percent change"""

    AnnualyPc = 8
    """The percent change from year ago"""


class IntrinioEconomicData(QuantConnect.Data.BaseData):
    """Access the massive repository of economic data from the Federal Reserve Economic Data system via the Intrinio API."""

    @typing.overload
    def __init__(self) -> None:
        """Initializes a new instance of the IntrinioEconomicData class."""
        ...

    @typing.overload
    def __init__(self, dataTransformation: QuantConnect.Data.Custom.Intrinio.IntrinioDataTransformation) -> None:
        """
        Initializes a new instance of the IntrinioEconomicData class.
        
        :param dataTransformation: The item.
        """
        ...

    def GetSource(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, isLiveMode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Return the URL string source of the file. This will be converted to a stream
        
        :param config: Configuration object
        :param date: Date of this source file
        :param isLiveMode: true if we're in live mode, false for backtesting mode
        :returns: String URL of source file.
        """
        ...

    def Reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, isLiveMode: bool) -> QuantConnect.Data.BaseData:
        """
        Reader converts each line of the data source into BaseData objects. Each data type creates its own factory method,
            and returns a new instance of the object
            each time it is called. The returned object is assumed to be time stamped in the config.ExchangeTimeZone.
        
        :param config: Subscription data config setup object
        :param line: Line of the source document
        :param date: Date of the requested data
        :param isLiveMode: true if we're in live mode, false for backtesting mode
        :returns: Instance of the T:BaseData object generated by this line of the CSV.
        """
        ...


class IntrinioEconomicDataSources(System.Object):
    """Intrinio Data Source"""

    class BofAMerrillLynch(System.Object):
        """Bank of America Merrill Lynch"""

        USCorporateBBBEffectiveYield: str = "$BAMLC0A4CBBBEY"
        """
        This data represents the effective yield of the BofA Merrill Lynch US Corporate BBB Index, a subset of the BofA
            Merrill Lynch US Corporate Master Index tracking the performance of US dollar denominated investment grade rated
            corporate debt publically issued in the US domestic market.
        """

        USCorporateBBBOptionAdjustedSpread: str = "$BAMLC0A4CBBB"
        """
        This data represents the Option-Adjusted Spread (OAS) of the BofA Merrill Lynch US Corporate BBB Index, a subset of
            the BofA Merrill Lynch US Corporate Master Index tracking the performance of US dollar denominated investment grade
            rated corporate debt publically issued in the US domestic market.
        """

        USCorporateMasterOptionAdjustedSpread: str = "$BAMLC0A0CM"
        """
        The BofA Merrill Lynch Option-Adjusted Spreads (OASs) are the calculated spreads between a computed OAS index of
            all bonds in a given rating category and a spot Treasury curve. An OAS index is constructed using each constituent
            bond’s OAS, weighted by market capitalization.
        """

        USHighYieldBBOptionAdjustedSpread: str = "$BAMLH0A1HYBB"
        """
        This data represents the Option-Adjusted Spread (OAS) of the BofA Merrill Lynch US Corporate BB Index, a subset of
            the BofA Merrill Lynch US High Yield Master II Index tracking the performance of US dollar denominated below
            investment grade rated corporate debt publically issued in the US domestic market.
        """

        USHighYieldBOptionAdjustedSpread: str = "$BAMLH0A2HYB"
        """
        This data represents the Option-Adjusted Spread (OAS) of the BofA Merrill Lynch US Corporate B Index, a subset of
            the BofA Merrill Lynch US High Yield Master II Index tracking the performance of US dollar denominated below
            investment grade rated corporate debt publically issued in the US domestic market. This subset includes all
            securities with a given investment grade rating B.
        """

        USHighYieldCCCorBelowOptionAdjustedSpread: str = "$BAMLH0A3HYC"
        """
        This data represents the Option-Adjusted Spread (OAS) of the BofA Merrill Lynch US Corporate C Index, a subset of
            the BofA Merrill Lynch US High Yield Master II Index tracking the performance of US dollar denominated below
            investment grade rated corporate debt publically issued in the US domestic market.
        """

        USHighYieldEffectiveYield: str = "$BAMLH0A0HYM2EY"
        """
        This data represents the effective yield of the BofA Merrill Lynch US High Yield Master II Index, which tracks the
            performance of US dollar denominated below investment grade rated corporate debt publically issued in the US
            domestic market.
            Source: https://fred.stlouisfed.org/series/BAMLH0A0HYM2EY
        """

        USHighYieldMasterIITotalReturnIndexValue: str = "$BAMLHYH0A0HYM2TRIV"
        """
        This data represents the BofA Merrill Lynch US High Yield Master II Index value, which tracks the performance of US
            dollar denominated below investment grade rated corporate debt publically issued in the US domestic market.
        """

        USHighYieldOptionAdjustedSpread: str = "$BAMLH0A0HYM2"
        """
        The BofA Merrill Lynch Option-Adjusted Spreads (OASs) are the calculated spreads between a computed OAS index of
            all bonds in a given rating category and a spot Treasury curve. An OAS index is constructed using each constituent
            bond’s OAS, weighted by market capitalization.
            Source: https://fred.stlouisfed.org/series/BAMLH0A0HYM2
        """

    class CBOE(System.Object):
        """Chicago Board Options Exchange"""

        ChinaETFVolatilityIndex: str = "$VXFXICLS"
        """CBOE China ETF Volatility Index"""

        CrudeOilETFVolatilityIndex: str = "$OVXCLS"
        """CBOE Crude Oil ETF Volatility Index"""

        EmergingMarketsETFVolatilityIndex: str = "$VXEEMCLS"
        """CBOE Emerging Markets ETF Volatility Index"""

        GoldETFVolatilityIndex: str = "$GVZCLS"
        """CBOE Gold ETF Volatility Index"""

        TenYearTreasuryNoteVolatilityFutures: str = "$VXTYN"
        """CBOE 10-Year Treasury Note Volatility Futures"""

        VIX: str = "$VIXCLS"
        """CBOE Volatility Index: VIX"""

        VXO: str = "$VXOCLS"
        """CBOE S&P 100 Volatility Index: VXO"""

        VXV: str = "$VXVCLS"
        """CBOE S&P 500 3-Month Volatility Index"""

    class Commodities(System.Object):
        """Commodities"""

        CrudeOilBrent: str = "$DCOILBRENTEU"
        """Crude Oil Prices: Brent - Europe"""

        CrudeOilWTI: str = "$DCOILWTICO"
        """Crude Oil Prices: West Texas Intermediate (WTI) - Cushing, Oklahoma"""

        GasolineUSGulfCoast: str = "$DGASUSGULF"
        """Conventional Gasoline Prices: U.S. Gulf Coast, Regular"""

        GoldFixingPrice1030amLondon: str = "$GOLDAMGBD228NLBM"
        """Gold Fixing Price 10:30 A.M. (London time) in London Bullion Market, based in U.S. Dollars"""

        GoldFixingPrice1500amLondon: str = "$GOLDPMGBD228NLBM"
        """Gold Fixing Price 3:00 P.M. (London time) in London Bullion Market, based in U.S. Dollars"""

        NaturalGas: str = "$DHHNGSP"
        """Henry Hub Natural Gas Spot Price"""

        Propane: str = "$DPROPANEMBTX"
        """Propane Prices: Mont Belvieu, Texas"""

    class ExchangeRates(System.Object):
        """Exchange Rates"""

        Brazil_USA: str = "$DEXBZUS"
        """Brazilian Reals to One U.S. Dollar"""

        Canada_USA: str = "$DEXCAUS"
        """Canadian Dollars to One U.S. Dollar"""

        China_USA: str = "$DEXCHUS"
        """Chinese Yuan to One U.S. Dollar"""

        HongKong_USA: str = "$DEXHKUS"
        """Hong Kong Dollars to One U.S. Dollar"""

        India_USA: str = "$DEXINUS"
        """Indian Rupees to One U.S. Dollar"""

        Japan_USA: str = "$DEXJPUS"
        """Japanese Yen to One U.S. Dollar"""

        Malaysia_USA: str = "$DEXMAUS"
        """Malaysian Ringgit to One U.S. Dollar"""

        Mexico_USA: str = "$DEXMXUS"
        """Mexican New Pesos to One U.S. Dollar"""

        Norway_USA: str = "$DEXNOUS"
        """Norwegian Kroner to One U.S. Dollar"""

        Singapore_USA: str = "$DEXSIUS"
        """Singapore Dollars to One U.S. Dollar"""

        SouthAfrica_USA: str = "$DEXSFUS"
        """South African Rand to One U.S. Dollar"""

        SouthKorea_USA: str = "$DEXKOUS"
        """South Korean Won to One U.S. Dollar"""

        SriLanka_USA: str = "$DEXSLUS"
        """Sri Lankan Rupees to One U.S. Dollar"""

        Switzerland_USA: str = "$DEXSZUS"
        """Swiss Francs to One U.S. Dollar"""

        Taiwan_USA: str = "$DEXTAUS"
        """New Taiwan Dollars to One U.S. Dollar"""

        Thailand_USA: str = "$DEXTHUS"
        """Thai Baht to One U.S. Dollar"""

        USA_Australia: str = "$DEXUSAL"
        """U.S. Dollars to One Australian Dollar"""

        USA_Euro: str = "$DEXUSEU"
        """U.S. Dollars to One Euro"""

        USA_NewZealand: str = "$DEXUSNZ"
        """U.S. Dollars to One New Zealand Dollar"""

        USA_UK: str = "$DEXUSUK"
        """U.S. Dollars to One British Pound"""

    class Moodys(System.Object):
        """Moody's Investors Service"""

        SeasonedAaaCorporateBondYield: str = "$DAAA"
        """
        Moody's Seasoned Aaa Corporate Bond© and 10-Year Treasury Constant Maturity.
            These instruments are based on bonds with maturities 20 years and above.
        """

        SeasonedAaaCorporateBondYieldRelativeTo10YearTreasuryConstantMaturity: str = "$AAA10Y"
        """
        Series is calculated as the spread between Moody's Seasoned Aaa Corporate Bond© and 10-Year Treasury Constant
            Maturity
        """

        SeasonedBaaCorporateBondYield: str = "$DBAA"
        """
        Moody's Seasoned Baa Corporate Bond© and 10-Year Treasury Constant Maturity.
            These instruments are based on bonds with maturities 20 years and above.
        """

        SeasonedBaaCorporateBondYieldRelativeTo10YearTreasuryConstantMaturity: str = "$BAA10Y"
        """Series is calculated as the spread between Moody's Seasoned Baa Corporate Bond© and 10-Year Treasury Constant Maturity"""

    class TradeWeightedUsDollarIndex(System.Object):
        """Trade Weighted US Dollar Index"""

        Broad: str = "$DTWEXB"
        """
        A weighted average of the foreign exchange value of the U.S. dollar against the currencies of a broad group of
            major U.S. trading partners. Broad currency index includes the Euro Area, Canada, Japan, Mexico, China, United
            Kingdom, Taiwan, Korea, Singapore, Hong Kong, Malaysia, Brazil, Switzerland, Thailand, Philippines, Australia,
            Indonesia, India, Israel, Saudi Arabia, Russia, Sweden, Argentina, Venezuela, Chile and Colombia.
        """

        MajorCurrencies: str = "$DTWEXM"
        """
        A weighted average of the foreign exchange value of the U.S. dollar against a subset of the broad index currencies
            that circulate widely outside the country of issue. Major currencies index includes the Euro Area, Canada, Japan,
            United Kingdom, Switzerland, Australia, and Sweden.
        """

        OtherImportantTradingPartners: str = "$DTWEXO"
        """
        A weighted average of the foreign exchange value of the U.S. dollar against a subset of the broad index currencies
            that do not circulate widely outside the country of issue. Countries whose currencies are included in the other
            important trading partners index are Mexico, China, Taiwan, Korea, Singapore, Hong Kong, Malaysia, Brazil,
            Thailand, Philippines, Indonesia, India, Israel, Saudi Arabia, Russia, Argentina, Venezuela, Chile and Colombia.
        """


class IntrinioConfig(System.Object):
    """Auxiliary class to access all Intrinio API data."""

    RateGate: QuantConnect.Util.RateGate = ...
    """"""

    IsInitialized: bool
    """Check if Intrinio API user and password are not empty or null."""

    Password: str = ...
    """Intrinio API password"""

    User: str = ...
    """Intrinio API user"""

    @staticmethod
    def SetTimeIntervalBetweenCalls(timeSpan: datetime.timedelta) -> None:
        """
        Sets the time interval between calls.
        For more information, please refer to: https://intrinio.com/documentation/api#limits
        
        :param timeSpan: Time interval between to consecutive calls.
        """
        ...

    @staticmethod
    def SetUserAndPassword(user: str, password: str) -> None:
        """Set the Intrinio API user and password."""
        ...





import datetime

import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Custom.CBOE
import QuantConnect.Data.Market
import System.Collections.Generic


class CBOE(QuantConnect.Data.Market.TradeBar):
    """CBOE data source"""

    def __init__(self) -> None:
        """Creates a new instance of the object"""
        ...

    def GetSource(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, isLiveMode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """Gets the source location of the CBOE file"""
        ...

    def Reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, isLiveMode: bool) -> QuantConnect.Data.BaseData:
        """
        Reads the data from the source and creates a BaseData instance
        
        :param config: Configuration
        :param line: Line of data
        :param date: Date we're requesting data for
        :param isLiveMode: Is live mode
        :returns: New BaseData instance to be used in the algorithm.
        """
        ...

    def RequiresMapping(self) -> bool:
        """
        Determines whether the data source requires mapping
        
        :returns: false.
        """
        ...

    def IsSparseData(self) -> bool:
        """
        Determines if data source is sparse
        
        :returns: false.
        """
        ...

    def ToString(self) -> str:
        """
        Converts the instance to a string
        
        :returns: String containing open, high, low, close.
        """
        ...

    def DefaultResolution(self) -> int:
        """
        Gets the default resolution for this data and security type
        
        :returns: This method returns the int value of a member of the QuantConnect.Resolution enum.
        """
        ...

    def SupportedResolutions(self) -> System.Collections.Generic.List[QuantConnect.Resolution]:
        """Gets the supported resolution for this data and security type"""
        ...





import abc
import datetime
import typing

import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.Results
import QuantConnect.Lean.Engine.TransactionHandlers
import QuantConnect.Orders
import QuantConnect.Securities
import System
import System.Collections.Concurrent
import System.Collections.Generic

System_EventHandler = typing.Any


class ITransactionHandler(QuantConnect.Securities.IOrderProcessor, QuantConnect.Securities.IOrderEventProvider, metaclass=abc.ABCMeta):
    """
    Transaction handlers define how the transactions are processed and set the order fill information.
    The pass this information back to the algorithm portfolio and ensure the cash and portfolio are synchronized.
    """

    @property
    @abc.abstractmethod
    def IsActive(self) -> bool:
        """
        Boolean flag indicating the thread is busy.
        False indicates it is completely finished processing and ready to be terminated.
        """
        ...

    @property
    @abc.abstractmethod
    def Orders(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.Order]:
        """Gets the permanent storage for all orders"""
        ...

    @property
    @abc.abstractmethod
    def OrderEvents(self) -> System.Collections.Generic.IEnumerable[QuantConnect.Orders.OrderEvent]:
        """Gets all order events"""
        ...

    @property
    @abc.abstractmethod
    def OrderTickets(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.OrderTicket]:
        """Gets the permanent storage for all order tickets"""
        ...

    def Initialize(self, algorithm: QuantConnect.Interfaces.IAlgorithm, brokerage: QuantConnect.Interfaces.IBrokerage, resultHandler: QuantConnect.Lean.Engine.Results.IResultHandler) -> None:
        """Initializes the transaction handler for the specified algorithm using the specified brokerage implementation"""
        ...

    def Exit(self) -> None:
        """Signal a end of thread request to stop montioring the transactions."""
        ...

    def ProcessSynchronousEvents(self) -> None:
        """Process any synchronous events from the primary algorithm thread."""
        ...

    def AddOpenOrder(self, order: QuantConnect.Orders.Order, orderTicket: QuantConnect.Orders.OrderTicket) -> None:
        """Register an already open Order"""
        ...


class CancelPendingOrders(System.Object):
    """Class used to keep track of CancelPending orders and their original or updated status"""

    @property
    def GetCancelPendingOrdersSize(self) -> int:
        """Amount of CancelPending Orders"""
        ...

    def Set(self, orderId: int, status: QuantConnect.Orders.OrderStatus) -> None:
        """
        Adds an order which will be canceled and we want to keep track of it Status in case of fallback
        
        :param orderId: The order id
        :param status: The order Status, before the cancel request
        """
        ...

    def UpdateOrRemove(self, orderId: int, newStatus: QuantConnect.Orders.OrderStatus) -> None:
        """
        Updates an order that is pending to be canceled.
        
        :param orderId: The id of the order
        :param newStatus: The new status of the order. If its OrderStatus.Canceled or OrderStatus.Filled it will be removed
        """
        ...

    def RemoveAndFallback(self, order: QuantConnect.Orders.Order) -> None:
        """
        Removes an order which we failed to cancel and falls back the order Status to previous value
        
        :param order: The order that failed to be canceled
        """
        ...


class BrokerageTransactionHandler(System.Object, QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler):
    """Transaction handler for all brokerages"""

    @property
    def _orderRequestQueue(self) -> QuantConnect.Interfaces.IBusyCollection[QuantConnect.Orders.OrderRequest]:
        """
        OrderQueue holds the newly updated orders from the user algorithm waiting to be processed. Once
        orders are processed they are moved into the Orders queue awaiting the brokerage response.
        
        This field is protected.
        """
        ...

    @_orderRequestQueue.setter
    def _orderRequestQueue(self, value: QuantConnect.Interfaces.IBusyCollection[QuantConnect.Orders.OrderRequest]):
        """
        OrderQueue holds the newly updated orders from the user algorithm waiting to be processed. Once
        orders are processed they are moved into the Orders queue awaiting the brokerage response.
        
        This field is protected.
        """
        ...

    @property
    def _cancelPendingOrders(self) -> QuantConnect.Lean.Engine.TransactionHandlers.CancelPendingOrders:
        """
        The _cancelPendingOrders instance will help to keep track of CancelPending orders and their Status
        
        This field is protected.
        """
        ...

    @property
    def NewOrderEvent(self) -> typing.List[System_EventHandler]:
        """Event fired when there is a new OrderEvent"""
        ...

    @NewOrderEvent.setter
    def NewOrderEvent(self, value: typing.List[System_EventHandler]):
        """Event fired when there is a new OrderEvent"""
        ...

    @property
    def Orders(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.Order]:
        """Gets the permanent storage for all orders"""
        ...

    @property
    def OrderEvents(self) -> System.Collections.Generic.IEnumerable[QuantConnect.Orders.OrderEvent]:
        """Gets all order events"""
        ...

    @property
    def OrderTickets(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.OrderTicket]:
        """Gets the permanent storage for all order tickets"""
        ...

    @property
    def OrdersCount(self) -> int:
        """Gets the current number of orders that have been processed"""
        ...

    @property
    def IsActive(self) -> bool:
        """
        Boolean flag indicating the Run thread method is busy.
        False indicates it is completely finished processing and ready to be terminated.
        """
        ...

    @IsActive.setter
    def IsActive(self, value: bool):
        """
        Boolean flag indicating the Run thread method is busy.
        False indicates it is completely finished processing and ready to be terminated.
        """
        ...

    @property
    def TimeSinceLastFill(self) -> datetime.timedelta:
        """
        Gets the amount of time since the last call to algorithm.Portfolio.ProcessFill(fill)
        
        This property is protected.
        """
        ...

    @property
    def CurrentTimeUtc(self) -> datetime.datetime:
        """
        Gets current time UTC. This is here to facilitate testing
        
        This property is protected.
        """
        ...

    def Initialize(self, algorithm: QuantConnect.Interfaces.IAlgorithm, brokerage: QuantConnect.Interfaces.IBrokerage, resultHandler: QuantConnect.Lean.Engine.Results.IResultHandler) -> None:
        """
        Creates a new BrokerageTransactionHandler to process orders using the specified brokerage implementation
        
        :param algorithm: The algorithm instance
        :param brokerage: The brokerage implementation to process orders and fire fill events
        """
        ...

    def InitializeTransactionThread(self) -> None:
        """
        Create and start the transaction thread, who will be in charge of processing
        the order requests
        
        This method is protected.
        """
        ...

    def Process(self, request: QuantConnect.Orders.OrderRequest) -> QuantConnect.Orders.OrderTicket:
        ...

    def AddOrder(self, request: QuantConnect.Orders.SubmitOrderRequest) -> QuantConnect.Orders.OrderTicket:
        """
        Add an order to collection and return the unique order id or negative if an error.
        
        :param request: A request detailing the order to be submitted
        :returns: New unique, increasing orderid.
        """
        ...

    def WaitForOrderSubmission(self, ticket: QuantConnect.Orders.OrderTicket) -> None:
        """
        Wait for the order to be handled by the _processingThread
        
        This method is protected.
        
        :param ticket: The OrderTicket expecting to be submitted
        """
        ...

    def UpdateOrder(self, request: QuantConnect.Orders.UpdateOrderRequest) -> QuantConnect.Orders.OrderTicket:
        """
        Update an order yet to be filled such as stop or limit orders.
        
        :param request: Request detailing how the order should be updated
        """
        ...

    def CancelOrder(self, request: QuantConnect.Orders.CancelOrderRequest) -> QuantConnect.Orders.OrderTicket:
        """
        Remove this order from outstanding queue: user is requesting a cancel.
        
        :param request: Request containing the specific order id to remove
        """
        ...

    def GetOrderTickets(self, filter: typing.Callable[[QuantConnect.Orders.OrderTicket], bool] = None) -> System.Collections.Generic.IEnumerable[QuantConnect.Orders.OrderTicket]:
        """
        Gets and enumerable of OrderTicket matching the specified
        
        :param filter: The filter predicate used to find the required order tickets
        :returns: An enumerable of OrderTicket matching the specified.
        """
        ...

    def GetOpenOrderTickets(self, filter: typing.Callable[[QuantConnect.Orders.OrderTicket], bool] = None) -> System.Collections.Generic.IEnumerable[QuantConnect.Orders.OrderTicket]:
        """
        Gets and enumerable of opened OrderTicket matching the specified
        
        :param filter: The filter predicate used to find the required order tickets
        :returns: An enumerable of opened OrderTicket matching the specified.
        """
        ...

    def GetOrderTicket(self, orderId: int) -> QuantConnect.Orders.OrderTicket:
        """
        Gets the order ticket for the specified order id. Returns null if not found
        
        :param orderId: The order's id
        :returns: The order ticket with the specified id, or null if not found.
        """
        ...

    def GetOrderById(self, orderId: int) -> QuantConnect.Orders.Order:
        ...

    def GetOrderByBrokerageId(self, brokerageId: str) -> QuantConnect.Orders.Order:
        """
        Gets the order by its brokerage id
        
        :param brokerageId: The brokerage id to fetch
        :returns: The first order matching the brokerage id, or null if no match is found.
        """
        ...

    def GetOrders(self, filter: typing.Callable[[QuantConnect.Orders.Order], bool] = None) -> System.Collections.Generic.IEnumerable[QuantConnect.Orders.Order]:
        """
        Gets all orders matching the specified filter. Specifying null will return an enumerable
        of all orders.
        
        :param filter: Delegate used to filter the orders
        :returns: All orders this order provider currently holds by the specified filter.
        """
        ...

    def GetOpenOrders(self, filter: typing.Callable[[QuantConnect.Orders.Order], bool] = None) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """
        Gets open orders matching the specified filter
        
        :param filter: Delegate used to filter the orders
        :returns: All open orders this order provider currently holds.
        """
        ...

    def Run(self) -> None:
        """
        Primary thread entry point to launch the transaction thread.
        
        This method is protected.
        """
        ...

    def ProcessAsynchronousEvents(self) -> None:
        """Processes asynchronous events on the transaction handler's thread"""
        ...

    def ProcessSynchronousEvents(self) -> None:
        """Processes all synchronous events that must take place before the next time loop for the algorithm"""
        ...

    def AddOpenOrder(self, order: QuantConnect.Orders.Order, orderTicket: QuantConnect.Orders.OrderTicket) -> None:
        """Register an already open Order"""
        ...

    def Exit(self) -> None:
        """Signal a end of thread request to stop monitoring the transactions."""
        ...

    def HandleOrderRequest(self, request: QuantConnect.Orders.OrderRequest) -> None:
        """
        Handles a generic order request
        
        :param request: OrderRequest to be handled
        :returns: OrderResponse for request.
        """
        ...

    def RoundOffOrder(self, order: QuantConnect.Orders.Order, security: QuantConnect.Securities.Security) -> float:
        """Rounds off the order towards 0 to the nearest multiple of Lot Size"""
        ...

    def RoundOrderPrices(self, order: QuantConnect.Orders.Order, security: QuantConnect.Securities.Security) -> None:
        """
        Rounds the order prices to its security minimum price variation.
        
        This procedure is needed to meet brokerage precision requirements.
        
        This method is protected.
        """
        ...


class BacktestingTransactionHandler(QuantConnect.Lean.Engine.TransactionHandlers.BrokerageTransactionHandler):
    """This transaction handler is used for processing transactions during backtests"""

    def Initialize(self, algorithm: QuantConnect.Interfaces.IAlgorithm, brokerage: QuantConnect.Interfaces.IBrokerage, resultHandler: QuantConnect.Lean.Engine.Results.IResultHandler) -> None:
        """
        Creates a new BacktestingTransactionHandler using the BacktestingBrokerage
        
        :param algorithm: The algorithm instance
        :param brokerage: The BacktestingBrokerage
        """
        ...

    def ProcessSynchronousEvents(self) -> None:
        """Processes all synchronous events that must take place before the next time loop for the algorithm"""
        ...

    def ProcessAsynchronousEvents(self) -> None:
        """Processes asynchronous events on the transaction handler's thread"""
        ...

    def WaitForOrderSubmission(self, ticket: QuantConnect.Orders.OrderTicket) -> None:
        """
        For backtesting we will submit the order ourselves
        
        This method is protected.
        
        :param ticket: The OrderTicket expecting to be submitted
        """
        ...

    def InitializeTransactionThread(self) -> None:
        """
        For backtesting order requests will be processed by the algorithm thread
        sequentially at WaitForOrderSubmission and ProcessSynchronousEvents
        
        This method is protected.
        """
        ...





import datetime
import typing

import QuantConnect.Brokerages.Zerodha.Messages
import System
import System.Collections.Generic


class DepthItem:
    """Market depth item structure"""

    @property
    def Quantity(self) -> int:
        ...

    @Quantity.setter
    def Quantity(self, value: int):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Orders(self) -> int:
        ...

    @Orders.setter
    def Orders(self, value: int):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class Tick:
    """This class has no documentation."""

    @property
    def Mode(self) -> str:
        ...

    @Mode.setter
    def Mode(self, value: str):
        ...

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def Tradable(self) -> bool:
        ...

    @Tradable.setter
    def Tradable(self, value: bool):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    @property
    def LastQuantity(self) -> int:
        ...

    @LastQuantity.setter
    def LastQuantity(self, value: int):
        ...

    @property
    def AveragePrice(self) -> float:
        ...

    @AveragePrice.setter
    def AveragePrice(self, value: float):
        ...

    @property
    def Volume(self) -> int:
        ...

    @Volume.setter
    def Volume(self, value: int):
        ...

    @property
    def BuyQuantity(self) -> int:
        ...

    @BuyQuantity.setter
    def BuyQuantity(self, value: int):
        ...

    @property
    def SellQuantity(self) -> int:
        ...

    @SellQuantity.setter
    def SellQuantity(self, value: int):
        ...

    @property
    def Open(self) -> float:
        ...

    @Open.setter
    def Open(self, value: float):
        ...

    @property
    def High(self) -> float:
        ...

    @High.setter
    def High(self, value: float):
        ...

    @property
    def Low(self) -> float:
        ...

    @Low.setter
    def Low(self, value: float):
        ...

    @property
    def Close(self) -> float:
        ...

    @Close.setter
    def Close(self, value: float):
        ...

    @property
    def Change(self) -> float:
        ...

    @Change.setter
    def Change(self, value: float):
        ...

    @property
    def Bids(self) -> typing.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]:
        ...

    @Bids.setter
    def Bids(self, value: typing.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]):
        ...

    @property
    def Offers(self) -> typing.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]:
        ...

    @Offers.setter
    def Offers(self, value: typing.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]):
        ...

    @property
    def LastTradeTime(self) -> typing.Optional[datetime.datetime]:
        ...

    @LastTradeTime.setter
    def LastTradeTime(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def OI(self) -> int:
        ...

    @OI.setter
    def OI(self, value: int):
        ...

    @property
    def OIDayHigh(self) -> int:
        ...

    @OIDayHigh.setter
    def OIDayHigh(self, value: int):
        ...

    @property
    def OIDayLow(self) -> int:
        ...

    @OIDayLow.setter
    def OIDayLow(self, value: int):
        ...

    @property
    def Timestamp(self) -> typing.Optional[datetime.datetime]:
        ...

    @Timestamp.setter
    def Timestamp(self, value: typing.Optional[datetime.datetime]):
        ...


class Historical:
    """Historical structure"""

    @property
    def TimeStamp(self) -> datetime.datetime:
        ...

    @property
    def Open(self) -> float:
        ...

    @property
    def High(self) -> float:
        ...

    @property
    def Low(self) -> float:
        ...

    @property
    def Close(self) -> float:
        ...

    @property
    def Volume(self) -> int:
        ...

    @property
    def OI(self) -> int:
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class Holding:
    """Holding structure"""

    @property
    def Product(self) -> str:
        ...

    @Product.setter
    def Product(self, value: str):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    @property
    def CollateralQuantity(self) -> int:
        ...

    @CollateralQuantity.setter
    def CollateralQuantity(self, value: int):
        ...

    @property
    def PNL(self) -> float:
        ...

    @PNL.setter
    def PNL(self, value: float):
        ...

    @property
    def ClosePrice(self) -> float:
        ...

    @ClosePrice.setter
    def ClosePrice(self, value: float):
        ...

    @property
    def AveragePrice(self) -> float:
        ...

    @AveragePrice.setter
    def AveragePrice(self, value: float):
        ...

    @property
    def TradingSymbol(self) -> str:
        ...

    @TradingSymbol.setter
    def TradingSymbol(self, value: str):
        ...

    @property
    def CollateralType(self) -> str:
        ...

    @CollateralType.setter
    def CollateralType(self, value: str):
        ...

    @property
    def T1Quantity(self) -> int:
        ...

    @T1Quantity.setter
    def T1Quantity(self, value: int):
        ...

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def ISIN(self) -> str:
        ...

    @ISIN.setter
    def ISIN(self, value: str):
        ...

    @property
    def RealisedQuantity(self) -> int:
        ...

    @RealisedQuantity.setter
    def RealisedQuantity(self, value: int):
        ...

    @property
    def Quantity(self) -> int:
        ...

    @Quantity.setter
    def Quantity(self, value: int):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class AvailableMargin:
    """Available margin structure"""

    @property
    def AdHocMargin(self) -> float:
        ...

    @AdHocMargin.setter
    def AdHocMargin(self, value: float):
        ...

    @property
    def Cash(self) -> float:
        ...

    @Cash.setter
    def Cash(self, value: float):
        ...

    @property
    def Collateral(self) -> float:
        ...

    @Collateral.setter
    def Collateral(self, value: float):
        ...

    @property
    def IntradayPayin(self) -> float:
        ...

    @IntradayPayin.setter
    def IntradayPayin(self, value: float):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class UtilisedMargin:
    """Utilised margin structure"""

    @property
    def Debits(self) -> float:
        ...

    @Debits.setter
    def Debits(self, value: float):
        ...

    @property
    def Exposure(self) -> float:
        ...

    @Exposure.setter
    def Exposure(self, value: float):
        ...

    @property
    def M2MRealised(self) -> float:
        ...

    @M2MRealised.setter
    def M2MRealised(self, value: float):
        ...

    @property
    def M2MUnrealised(self) -> float:
        ...

    @M2MUnrealised.setter
    def M2MUnrealised(self, value: float):
        ...

    @property
    def OptionPremium(self) -> float:
        ...

    @OptionPremium.setter
    def OptionPremium(self, value: float):
        ...

    @property
    def Payout(self) -> float:
        ...

    @Payout.setter
    def Payout(self, value: float):
        ...

    @property
    def Span(self) -> float:
        ...

    @Span.setter
    def Span(self, value: float):
        ...

    @property
    def HoldingSales(self) -> float:
        ...

    @HoldingSales.setter
    def HoldingSales(self, value: float):
        ...

    @property
    def Turnover(self) -> float:
        ...

    @Turnover.setter
    def Turnover(self, value: float):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class UserMargin:
    """UserMargin structure"""

    @property
    def Enabled(self) -> bool:
        ...

    @Enabled.setter
    def Enabled(self, value: bool):
        ...

    @property
    def Net(self) -> float:
        ...

    @Net.setter
    def Net(self, value: float):
        ...

    @property
    def Available(self) -> QuantConnect.Brokerages.Zerodha.Messages.AvailableMargin:
        ...

    @Available.setter
    def Available(self, value: QuantConnect.Brokerages.Zerodha.Messages.AvailableMargin):
        ...

    @property
    def Utilised(self) -> QuantConnect.Brokerages.Zerodha.Messages.UtilisedMargin:
        ...

    @Utilised.setter
    def Utilised(self, value: QuantConnect.Brokerages.Zerodha.Messages.UtilisedMargin):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class UserMarginsResponse:
    """User margins response structure"""

    @property
    def Equity(self) -> QuantConnect.Brokerages.Zerodha.Messages.UserMargin:
        ...

    @Equity.setter
    def Equity(self, value: QuantConnect.Brokerages.Zerodha.Messages.UserMargin):
        ...

    @property
    def Commodity(self) -> QuantConnect.Brokerages.Zerodha.Messages.UserMargin:
        ...

    @Commodity.setter
    def Commodity(self, value: QuantConnect.Brokerages.Zerodha.Messages.UserMargin):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class InstrumentMargin:
    """UserMargin structure"""

    @property
    def Tradingsymbol(self) -> str:
        ...

    @Tradingsymbol.setter
    def Tradingsymbol(self, value: str):
        ...

    @property
    def Margin(self) -> float:
        ...

    @Margin.setter
    def Margin(self, value: float):
        ...

    @property
    def COLower(self) -> float:
        ...

    @COLower.setter
    def COLower(self, value: float):
        ...

    @property
    def COUpper(self) -> float:
        ...

    @COUpper.setter
    def COUpper(self, value: float):
        ...

    @property
    def MISMultiplier(self) -> float:
        ...

    @MISMultiplier.setter
    def MISMultiplier(self, value: float):
        ...

    @property
    def MISMargin(self) -> float:
        ...

    @MISMargin.setter
    def MISMargin(self, value: float):
        ...

    @property
    def NRMLMargin(self) -> float:
        ...

    @NRMLMargin.setter
    def NRMLMargin(self, value: float):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class Position:
    """Position structure"""

    @property
    def Product(self) -> str:
        ...

    @property
    def OvernightQuantity(self) -> int:
        ...

    @property
    def Exchange(self) -> str:
        ...

    @property
    def SellValue(self) -> float:
        ...

    @property
    def BuyM2M(self) -> float:
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @property
    def TradingSymbol(self) -> str:
        ...

    @property
    def Realised(self) -> float:
        ...

    @property
    def PNL(self) -> float:
        ...

    @property
    def Multiplier(self) -> float:
        ...

    @property
    def SellQuantity(self) -> int:
        ...

    @property
    def SellM2M(self) -> float:
        ...

    @property
    def BuyValue(self) -> float:
        ...

    @property
    def BuyQuantity(self) -> int:
        ...

    @property
    def AveragePrice(self) -> float:
        ...

    @property
    def Unrealised(self) -> float:
        ...

    @property
    def Value(self) -> float:
        ...

    @property
    def BuyPrice(self) -> float:
        ...

    @property
    def SellPrice(self) -> float:
        ...

    @property
    def M2M(self) -> float:
        ...

    @property
    def InstrumentToken(self) -> int:
        ...

    @property
    def ClosePrice(self) -> float:
        ...

    @property
    def Quantity(self) -> int:
        ...

    @property
    def DayBuyQuantity(self) -> int:
        ...

    @property
    def DayBuyPrice(self) -> float:
        ...

    @property
    def DayBuyValue(self) -> float:
        ...

    @property
    def DaySellQuantity(self) -> int:
        ...

    @property
    def DaySellPrice(self) -> float:
        ...

    @property
    def DaySellValue(self) -> float:
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class PositionResponse:
    """Position response structure"""

    @property
    def Day(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.Position]:
        ...

    @property
    def Net(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.Position]:
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class Order:
    """Order structure"""

    @property
    def AveragePrice(self) -> float:
        ...

    @AveragePrice.setter
    def AveragePrice(self, value: float):
        ...

    @property
    def CancelledQuantity(self) -> int:
        ...

    @CancelledQuantity.setter
    def CancelledQuantity(self, value: int):
        ...

    @property
    def DisclosedQuantity(self) -> int:
        ...

    @DisclosedQuantity.setter
    def DisclosedQuantity(self, value: int):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def ExchangeOrderId(self) -> str:
        ...

    @ExchangeOrderId.setter
    def ExchangeOrderId(self, value: str):
        ...

    @property
    def ExchangeTimestamp(self) -> typing.Optional[datetime.datetime]:
        ...

    @ExchangeTimestamp.setter
    def ExchangeTimestamp(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def FilledQuantity(self) -> int:
        ...

    @FilledQuantity.setter
    def FilledQuantity(self, value: int):
        ...

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def OrderId(self) -> str:
        ...

    @OrderId.setter
    def OrderId(self, value: str):
        ...

    @property
    def OrderTimestamp(self) -> typing.Optional[datetime.datetime]:
        ...

    @OrderTimestamp.setter
    def OrderTimestamp(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def OrderType(self) -> str:
        ...

    @OrderType.setter
    def OrderType(self, value: str):
        ...

    @property
    def ParentOrderId(self) -> str:
        ...

    @ParentOrderId.setter
    def ParentOrderId(self, value: str):
        ...

    @property
    def PendingQuantity(self) -> int:
        ...

    @PendingQuantity.setter
    def PendingQuantity(self, value: int):
        ...

    @property
    def PlacedBy(self) -> str:
        ...

    @PlacedBy.setter
    def PlacedBy(self, value: str):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Product(self) -> str:
        ...

    @Product.setter
    def Product(self, value: str):
        ...

    @property
    def Quantity(self) -> int:
        ...

    @Quantity.setter
    def Quantity(self, value: int):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def StatusMessage(self) -> str:
        ...

    @StatusMessage.setter
    def StatusMessage(self, value: str):
        ...

    @property
    def Tag(self) -> str:
        ...

    @Tag.setter
    def Tag(self, value: str):
        ...

    @property
    def Tradingsymbol(self) -> str:
        ...

    @Tradingsymbol.setter
    def Tradingsymbol(self, value: str):
        ...

    @property
    def TransactionType(self) -> str:
        ...

    @TransactionType.setter
    def TransactionType(self, value: str):
        ...

    @property
    def TriggerPrice(self) -> float:
        ...

    @TriggerPrice.setter
    def TriggerPrice(self, value: float):
        ...

    @property
    def Validity(self) -> str:
        ...

    @Validity.setter
    def Validity(self, value: str):
        ...

    @property
    def Variety(self) -> str:
        ...

    @Variety.setter
    def Variety(self, value: str):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class ChannelSubscription:
    """This class has no documentation."""

    @property
    def ChannelId(self) -> str:
        ...

    @ChannelId.setter
    def ChannelId(self, value: str):
        ...

    @property
    def a(self) -> str:
        ...

    @a.setter
    def a(self, value: str):
        ...

    @property
    def v(self) -> typing.List[int]:
        ...

    @v.setter
    def v(self, value: typing.List[int]):
        ...


class ChannelUnsubscription:
    """This class has no documentation."""

    @property
    def ChannelId(self) -> str:
        ...

    @ChannelId.setter
    def ChannelId(self, value: str):
        ...

    @property
    def a(self) -> str:
        ...

    @a.setter
    def a(self, value: str):
        ...

    @property
    def v(self) -> typing.List[int]:
        ...

    @v.setter
    def v(self, value: typing.List[int]):
        ...


class GTTCondition:
    """GTTCondition structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def TradingSymbol(self) -> str:
        ...

    @TradingSymbol.setter
    def TradingSymbol(self, value: str):
        ...

    @property
    def TriggerValues(self) -> System.Collections.Generic.List[float]:
        ...

    @TriggerValues.setter
    def TriggerValues(self, value: System.Collections.Generic.List[float]):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class GTTOrderResult:
    """GTTOrderResult structure"""

    @property
    def OrderId(self) -> str:
        ...

    @OrderId.setter
    def OrderId(self, value: str):
        ...

    @property
    def RejectionReason(self) -> str:
        ...

    @RejectionReason.setter
    def RejectionReason(self, value: str):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class GTTResult:
    """GTTResult structure"""

    @property
    def OrderResult(self) -> typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTOrderResult]:
        ...

    @OrderResult.setter
    def OrderResult(self, value: typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTOrderResult]):
        ...

    @property
    def Timestamp(self) -> str:
        ...

    @Timestamp.setter
    def Timestamp(self, value: str):
        ...

    @property
    def TriggeredAtPrice(self) -> float:
        ...

    @TriggeredAtPrice.setter
    def TriggeredAtPrice(self, value: float):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class GTTOrder:
    """GTTOrder structure"""

    @property
    def TransactionType(self) -> str:
        ...

    @TransactionType.setter
    def TransactionType(self, value: str):
        ...

    @property
    def Product(self) -> str:
        ...

    @Product.setter
    def Product(self, value: str):
        ...

    @property
    def OrderType(self) -> str:
        ...

    @OrderType.setter
    def OrderType(self, value: str):
        ...

    @property
    def Quantity(self) -> int:
        ...

    @Quantity.setter
    def Quantity(self, value: int):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def Result(self) -> typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTResult]:
        ...

    @Result.setter
    def Result(self, value: typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTResult]):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class GTTMeta:
    """GTTMeta structure"""

    @property
    def RejectionReason(self) -> str:
        ...

    @RejectionReason.setter
    def RejectionReason(self, value: str):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class GTT:
    """GTTOrder structure"""

    @property
    def Id(self) -> int:
        ...

    @Id.setter
    def Id(self, value: int):
        ...

    @property
    def Condition(self) -> typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTCondition]:
        ...

    @Condition.setter
    def Condition(self, value: typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTCondition]):
        ...

    @property
    def TriggerType(self) -> str:
        ...

    @TriggerType.setter
    def TriggerType(self, value: str):
        ...

    @property
    def Orders(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.GTTOrder]:
        ...

    @Orders.setter
    def Orders(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.GTTOrder]):
        ...

    @property
    def Status(self) -> str:
        ...

    @Status.setter
    def Status(self, value: str):
        ...

    @property
    def CreatedAt(self) -> typing.Optional[datetime.datetime]:
        ...

    @CreatedAt.setter
    def CreatedAt(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def UpdatedAt(self) -> typing.Optional[datetime.datetime]:
        ...

    @UpdatedAt.setter
    def UpdatedAt(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def ExpiresAt(self) -> typing.Optional[datetime.datetime]:
        ...

    @ExpiresAt.setter
    def ExpiresAt(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def Meta(self) -> typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTMeta]:
        ...

    @Meta.setter
    def Meta(self, value: typing.Optional[QuantConnect.Brokerages.Zerodha.Messages.GTTMeta]):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class GTTOrderParams:
    """GTTOrderParams structure"""

    @property
    def Quantity(self) -> int:
        ...

    @Quantity.setter
    def Quantity(self, value: int):
        ...

    @property
    def Price(self) -> float:
        ...

    @Price.setter
    def Price(self, value: float):
        ...

    @property
    def OrderType(self) -> str:
        ...

    @OrderType.setter
    def OrderType(self, value: str):
        ...

    @property
    def Product(self) -> str:
        ...

    @Product.setter
    def Product(self, value: str):
        ...

    @property
    def TransactionType(self) -> str:
        ...

    @TransactionType.setter
    def TransactionType(self, value: str):
        ...


class GTTParams:
    """GTTParams structure"""

    @property
    def TradingSymbol(self) -> str:
        ...

    @TradingSymbol.setter
    def TradingSymbol(self, value: str):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def TriggerType(self) -> str:
        ...

    @TriggerType.setter
    def TriggerType(self, value: str):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    @property
    def Orders(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.GTTOrderParams]:
        ...

    @Orders.setter
    def Orders(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.GTTOrderParams]):
        ...

    @property
    def TriggerPrices(self) -> System.Collections.Generic.List[float]:
        ...

    @TriggerPrices.setter
    def TriggerPrices(self, value: System.Collections.Generic.List[float]):
        ...


class Instrument:
    """Instrument structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def ExchangeToken(self) -> int:
        ...

    @ExchangeToken.setter
    def ExchangeToken(self, value: int):
        ...

    @property
    def TradingSymbol(self) -> str:
        ...

    @TradingSymbol.setter
    def TradingSymbol(self, value: str):
        ...

    @property
    def Name(self) -> str:
        ...

    @Name.setter
    def Name(self, value: str):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    @property
    def TickSize(self) -> float:
        ...

    @TickSize.setter
    def TickSize(self, value: float):
        ...

    @property
    def Expiry(self) -> typing.Optional[datetime.datetime]:
        ...

    @Expiry.setter
    def Expiry(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def InstrumentType(self) -> str:
        ...

    @InstrumentType.setter
    def InstrumentType(self, value: str):
        ...

    @property
    def Segment(self) -> str:
        ...

    @Segment.setter
    def Segment(self, value: str):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def Strike(self) -> float:
        ...

    @Strike.setter
    def Strike(self, value: float):
        ...

    @property
    def LotSize(self) -> int:
        ...

    @LotSize.setter
    def LotSize(self, value: int):
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class CsvInstrument(System.Object):
    """Instrument structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def ExchangeToken(self) -> int:
        ...

    @ExchangeToken.setter
    def ExchangeToken(self, value: int):
        ...

    @property
    def TradingSymbol(self) -> str:
        ...

    @TradingSymbol.setter
    def TradingSymbol(self, value: str):
        ...

    @property
    def Name(self) -> str:
        ...

    @Name.setter
    def Name(self, value: str):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    @property
    def TickSize(self) -> float:
        ...

    @TickSize.setter
    def TickSize(self, value: float):
        ...

    @property
    def Expiry(self) -> typing.Optional[datetime.datetime]:
        ...

    @Expiry.setter
    def Expiry(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def InstrumentType(self) -> str:
        ...

    @InstrumentType.setter
    def InstrumentType(self, value: str):
        ...

    @property
    def Segment(self) -> str:
        ...

    @Segment.setter
    def Segment(self, value: str):
        ...

    @property
    def Exchange(self) -> str:
        ...

    @Exchange.setter
    def Exchange(self, value: str):
        ...

    @property
    def Strike(self) -> float:
        ...

    @Strike.setter
    def Strike(self, value: float):
        ...

    @property
    def LotSize(self) -> int:
        ...

    @LotSize.setter
    def LotSize(self, value: int):
        ...


class Trade:
    """Trade structure"""

    @property
    def TradeId(self) -> str:
        ...

    @property
    def OrderId(self) -> str:
        ...

    @property
    def ExchangeOrderId(self) -> str:
        ...

    @property
    def Tradingsymbol(self) -> str:
        ...

    @property
    def Exchange(self) -> str:
        ...

    @property
    def InstrumentToken(self) -> int:
        ...

    @property
    def TransactionType(self) -> str:
        ...

    @property
    def Product(self) -> str:
        ...

    @property
    def AveragePrice(self) -> float:
        ...

    @property
    def Quantity(self) -> int:
        ...

    @property
    def FillTimestamp(self) -> typing.Optional[datetime.datetime]:
        ...

    @property
    def ExchangeTimestamp(self) -> typing.Optional[datetime.datetime]:
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class TrigerRange:
    """Trigger range structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @property
    def Lower(self) -> float:
        ...

    @property
    def Upper(self) -> float:
        ...

    @property
    def Percentage(self) -> float:
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class User:
    """User structure"""

    @property
    def APIKey(self) -> str:
        ...

    @property
    def Products(self) -> typing.List[str]:
        ...

    @property
    def UserName(self) -> str:
        ...

    @property
    def UserShortName(self) -> str:
        ...

    @property
    def AvatarURL(self) -> str:
        ...

    @property
    def Broker(self) -> str:
        ...

    @property
    def AccessToken(self) -> str:
        ...

    @property
    def PublicToken(self) -> str:
        ...

    @property
    def RefreshToken(self) -> str:
        ...

    @property
    def UserType(self) -> str:
        ...

    @property
    def UserId(self) -> str:
        ...

    @property
    def LoginTime(self) -> typing.Optional[datetime.datetime]:
        ...

    @property
    def Exchanges(self) -> typing.List[str]:
        ...

    @property
    def OrderTypes(self) -> typing.List[str]:
        ...

    @property
    def Email(self) -> str:
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class TokenSet:
    """This class has no documentation."""

    @property
    def UserId(self) -> str:
        ...

    @property
    def AccessToken(self) -> str:
        ...

    @property
    def RefreshToken(self) -> str:
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class Profile:
    """User structure"""

    @property
    def Products(self) -> typing.List[str]:
        ...

    @property
    def UserName(self) -> str:
        ...

    @property
    def UserShortName(self) -> str:
        ...

    @property
    def AvatarURL(self) -> str:
        ...

    @property
    def Broker(self) -> str:
        ...

    @property
    def UserType(self) -> str:
        ...

    @property
    def Exchanges(self) -> typing.List[str]:
        ...

    @property
    def OrderTypes(self) -> typing.List[str]:
        ...

    @property
    def Email(self) -> str:
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class Quote:
    """Quote structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @LastPrice.setter
    def LastPrice(self, value: float):
        ...

    @property
    def LastQuantity(self) -> int:
        ...

    @LastQuantity.setter
    def LastQuantity(self, value: int):
        ...

    @property
    def AveragePrice(self) -> float:
        ...

    @AveragePrice.setter
    def AveragePrice(self, value: float):
        ...

    @property
    def Volume(self) -> int:
        ...

    @Volume.setter
    def Volume(self, value: int):
        ...

    @property
    def BuyQuantity(self) -> int:
        ...

    @BuyQuantity.setter
    def BuyQuantity(self, value: int):
        ...

    @property
    def SellQuantity(self) -> int:
        ...

    @SellQuantity.setter
    def SellQuantity(self, value: int):
        ...

    @property
    def Open(self) -> float:
        ...

    @Open.setter
    def Open(self, value: float):
        ...

    @property
    def High(self) -> float:
        ...

    @High.setter
    def High(self, value: float):
        ...

    @property
    def Low(self) -> float:
        ...

    @Low.setter
    def Low(self, value: float):
        ...

    @property
    def Close(self) -> float:
        ...

    @Close.setter
    def Close(self, value: float):
        ...

    @property
    def Change(self) -> float:
        ...

    @Change.setter
    def Change(self, value: float):
        ...

    @property
    def LowerCircuitLimit(self) -> float:
        ...

    @LowerCircuitLimit.setter
    def LowerCircuitLimit(self, value: float):
        ...

    @property
    def UpperCircuitLimit(self) -> float:
        ...

    @UpperCircuitLimit.setter
    def UpperCircuitLimit(self, value: float):
        ...

    @property
    def Bids(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]:
        ...

    @Bids.setter
    def Bids(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]):
        ...

    @property
    def Offers(self) -> System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]:
        ...

    @Offers.setter
    def Offers(self, value: System.Collections.Generic.List[QuantConnect.Brokerages.Zerodha.Messages.DepthItem]):
        ...

    @property
    def LastTradeTime(self) -> typing.Optional[datetime.datetime]:
        ...

    @LastTradeTime.setter
    def LastTradeTime(self, value: typing.Optional[datetime.datetime]):
        ...

    @property
    def OI(self) -> int:
        ...

    @OI.setter
    def OI(self, value: int):
        ...

    @property
    def OIDayHigh(self) -> int:
        ...

    @OIDayHigh.setter
    def OIDayHigh(self, value: int):
        ...

    @property
    def OIDayLow(self) -> int:
        ...

    @OIDayLow.setter
    def OIDayLow(self, value: int):
        ...

    @property
    def Timestamp(self) -> typing.Optional[datetime.datetime]:
        ...

    @Timestamp.setter
    def Timestamp(self, value: typing.Optional[datetime.datetime]):
        ...

    def __init__(self, data: typing.Any) -> None:
        ...


class OHLC:
    """OHLC Quote structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    @property
    def Open(self) -> float:
        ...

    @property
    def Close(self) -> float:
        ...

    @property
    def High(self) -> float:
        ...

    @property
    def Low(self) -> float:
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...


class LTP:
    """LTP Quote structure"""

    @property
    def InstrumentToken(self) -> int:
        ...

    @InstrumentToken.setter
    def InstrumentToken(self, value: int):
        ...

    @property
    def LastPrice(self) -> float:
        ...

    def __init__(self, data: System.Collections.Generic.Dictionary[str, typing.Any]) -> None:
        ...





import abc
import typing

import QuantConnect
import QuantConnect.Algorithm
import QuantConnect.Algorithm.Framework
import QuantConnect.Algorithm.Framework.Execution
import QuantConnect.Algorithm.Framework.Portfolio
import QuantConnect.Data.Consolidators
import QuantConnect.Data.UniverseSelection
import QuantConnect.Indicators
import QuantConnect.Securities
import System


class IExecutionModel(QuantConnect.Algorithm.Framework.INotifiedSecurityChanges, metaclass=abc.ABCMeta):
    """Algorithm framework model that executes portfolio targets"""

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Submit orders for the specified portfolio targets.
        This model is free to delay or spread out these orders as it sees fit
        
        :param algorithm: The algorithm instance
        :param targets: The portfolio targets just emitted by the portfolio construction model. These are always just the new/updated targets and not a complete set of targets
        """
        ...


class ExecutionModel(System.Object, QuantConnect.Algorithm.Framework.Execution.IExecutionModel):
    """Provides a base class for execution models"""

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Submit orders for the specified portolio targets.
        This model is free to delay or spread out these orders as it sees fit
        
        :param algorithm: The algorithm instance
        :param targets: The portfolio targets just emitted by the portfolio construction model. These are always just the new/updated targets and not a complete set of targets
        """
        ...

    def OnSecuritiesChanged(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time the we add/remove securities from the data feed
        
        :param algorithm: The algorithm instance that experienced the change in securities
        :param changes: The security additions and removals from the algorithm
        """
        ...


class StandardDeviationExecutionModel(QuantConnect.Algorithm.Framework.Execution.ExecutionModel):
    """
    Execution model that submits orders while the current market prices is at least the configured number of standard
    deviations away from the mean in the favorable direction (below/above for buy/sell respectively)
    """

    class SymbolData(System.Object):
        """
        Symbol Data for this Execution Model
        
        This class is protected.
        """

        @property
        def Security(self) -> QuantConnect.Securities.Security:
            """Security"""
            ...

        @property
        def STD(self) -> QuantConnect.Indicators.StandardDeviation:
            """Standard Deviation"""
            ...

        @property
        def SMA(self) -> QuantConnect.Indicators.SimpleMovingAverage:
            """Simple Moving Average"""
            ...

        @property
        def Consolidator(self) -> QuantConnect.Data.Consolidators.IDataConsolidator:
            """Data Consolidator"""
            ...

        def __init__(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, security: QuantConnect.Securities.Security, period: int, resolution: QuantConnect.Resolution) -> None:
            """
            Initialize an instance of SymbolData
            
            :param algorithm: Algorithm for this security
            :param security: The security we are using
            :param period: Period of the SMA and STD
            :param resolution: Resolution for this symbol
            """
            ...

    @property
    def MaximumOrderValue(self) -> float:
        """
        Gets or sets the maximum order value in units of the account currency.
        This defaults to $20,000. For example, if purchasing a stock with a price
        of $100, then the maximum order size would be 200 shares.
        """
        ...

    @MaximumOrderValue.setter
    def MaximumOrderValue(self, value: float):
        """
        Gets or sets the maximum order value in units of the account currency.
        This defaults to $20,000. For example, if purchasing a stock with a price
        of $100, then the maximum order size would be 200 shares.
        """
        ...

    def __init__(self, period: int = 60, deviations: float = 2, resolution: QuantConnect.Resolution = ...) -> None:
        """
        Initializes a new instance of the StandardDeviationExecutionModel class
        
        :param period: Period of the standard deviation indicator
        :param deviations: The number of deviations away from the mean before submitting an order
        :param resolution: The resolution of the STD and SMA indicators
        """
        ...

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Executes market orders if the standard deviation of price is more than the configured number of deviations
        in the favorable direction.
        
        :param algorithm: The algorithm instance
        :param targets: The portfolio targets
        """
        ...

    def OnSecuritiesChanged(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time the we add/remove securities from the data feed
        
        :param algorithm: The algorithm instance that experienced the change in securities
        :param changes: The security additions and removals from the algorithm
        """
        ...

    def PriceIsFavorable(self, data: QuantConnect.Algorithm.Framework.Execution.StandardDeviationExecutionModel.SymbolData, unorderedQuantity: float) -> bool:
        """
        Determines if the current price is more than the configured number of standard deviations
        away from the mean in the favorable direction.
        
        This method is protected.
        """
        ...

    def IsSafeToRemove(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """
        Determines if it's safe to remove the associated symbol data
        
        This method is protected.
        """
        ...


class VolumeWeightedAveragePriceExecutionModel(QuantConnect.Algorithm.Framework.Execution.ExecutionModel):
    """Execution model that submits orders while the current market price is more favorable that the current volume weighted average price."""

    class SymbolData(System.Object):
        """
        Symbol data for this Execution Model
        
        This class is protected.
        """

        @property
        def Security(self) -> QuantConnect.Securities.Security:
            """Security"""
            ...

        @property
        def VWAP(self) -> QuantConnect.Indicators.IntradayVwap:
            """VWAP Indicator"""
            ...

        @property
        def Consolidator(self) -> QuantConnect.Data.Consolidators.IDataConsolidator:
            """Data Consolidator"""
            ...

        def __init__(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, security: QuantConnect.Securities.Security) -> None:
            """Initialize a new instance of SymbolData"""
            ...

    @property
    def MaximumOrderQuantityPercentVolume(self) -> float:
        """
        Gets or sets the maximum order quantity as a percentage of the current bar's volume.
        This defaults to 0.01m = 1%. For example, if the current bar's volume is 100, then
        the maximum order size would equal 1 share.
        """
        ...

    @MaximumOrderQuantityPercentVolume.setter
    def MaximumOrderQuantityPercentVolume(self, value: float):
        """
        Gets or sets the maximum order quantity as a percentage of the current bar's volume.
        This defaults to 0.01m = 1%. For example, if the current bar's volume is 100, then
        the maximum order size would equal 1 share.
        """
        ...

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Submit orders for the specified portolio targets.
        This model is free to delay or spread out these orders as it sees fit
        
        :param algorithm: The algorithm instance
        :param targets: The portfolio targets to be ordered
        """
        ...

    def OnSecuritiesChanged(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time the we add/remove securities from the data feed
        
        :param algorithm: The algorithm instance that experienced the change in securities
        :param changes: The security additions and removals from the algorithm
        """
        ...

    def IsSafeToRemove(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """
        Determines if it's safe to remove the associated symbol data
        
        This method is protected.
        """
        ...

    def PriceIsFavorable(self, data: QuantConnect.Algorithm.Framework.Execution.VolumeWeightedAveragePriceExecutionModel.SymbolData, unorderedQuantity: float) -> bool:
        """
        Determines if the current price is better than VWAP
        
        This method is protected.
        """
        ...


class ImmediateExecutionModel(QuantConnect.Algorithm.Framework.Execution.ExecutionModel):
    """
    Provides an implementation of IExecutionModel that immediately submits
    market orders to achieve the desired portfolio targets
    """

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Immediately submits orders for the specified portfolio targets.
        
        :param algorithm: The algorithm instance
        :param targets: The portfolio targets to be ordered
        """
        ...

    def OnSecuritiesChanged(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time the we add/remove securities from the data feed
        
        :param algorithm: The algorithm instance that experienced the change in securities
        :param changes: The security additions and removals from the algorithm
        """
        ...


class NullExecutionModel(QuantConnect.Algorithm.Framework.Execution.ExecutionModel):
    """Provides an implementation of IExecutionModel that does nothing"""

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Execute the ExecutionModel
        
        :param algorithm: The Algorithm to execute this model on
        :param targets: The portfolio targets
        """
        ...


class ExecutionModelPythonWrapper(QuantConnect.Algorithm.Framework.Execution.ExecutionModel):
    """Provides an implementation of IExecutionModel that wraps a PyObject object"""

    def __init__(self, model: typing.Any) -> None:
        """
        Constructor for initialising the IExecutionModel class with wrapped PyObject object
        
        :param model: Model defining how to execute trades to reach a portfolio target
        """
        ...

    def Execute(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, targets: typing.List[QuantConnect.Algorithm.Framework.Portfolio.IPortfolioTarget]) -> None:
        """
        Submit orders for the specified portolio targets.
        This model is free to delay or spread out these orders as it sees fit
        
        :param algorithm: The algorithm instance
        :param targets: The portfolio targets to be ordered
        """
        ...

    def OnSecuritiesChanged(self, algorithm: QuantConnect.Algorithm.QCAlgorithm, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time the we add/remove securities from the data feed
        
        :param algorithm: The algorithm instance that experienced the change in securities
        :param changes: The security additions and removals from the algorithm
        """
        ...


